--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2019 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
-- Communications Test - read up values using RTCEngine and check their value
title = "Communcations Test"
--
-- This test repeatedly reads several EEPROM values in the Engine and compares
-- the values returned with initialially read values.
--
-- We should be able to do this forever without any detected errors.
--
-- When you press STOP to end the test, the script prints out a "Loops per second"
-- value which can be used to determine the relative speed of the communications hardware
-- when run with different configurations (ie: WiFi vs RS232, panStamp vs Seeeduino, etc).
--
-- Typical results:
-- USB with Seeeduino : 0.74 loops/second
-- WiFi with ESP8266  : 1.03 loops/second
--
require("defines");
require("functions");

-- Counter Labels
Counter01Label = "Data Errors";
Counter02Label = "Comm Errors";
Counter03Label = "Loops";

Check024 = 15;
Check008 = 84;
Check009 = 133;
Check067 = 0;

DebugThreshold = 3;

PacketCount = 0;
PreviousCount = 0;
--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	-- the setup() function is called once at the beginning of the run
	print("setup(): Engine Number " .. Engine .. " TIU Number " .. TIU .. " Debug " .. Debug());
	-- setup check data
	Check024 = RAM(0x024, Engine, TIU);	-- q1F84	-- Status Byte 2
	print("RAM 0x024 = ", Check024);
	Check008 = RAM(0x008, Engine, TIU);	-- q3F84	-- Scaling Factor
	print("RAM 0x008 = ", Check008);
	Check009 = RAM(0x009, Engine, TIU);	-- q3FB3	-- Scaling Factor
	print("RAM 0x009 = ", Check009);
	Check067 = RAM(0x067, Engine, TIU);	-- q3F84	-- Direction
	print("RAM 0x067 = ", Check067);
	print("Runs quietly unless Debug is at least " .. DebugThreshold);
	return true;		-- false=setup failed, true=setup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
function loop(Engine, TIU)
	-- The loop() function is repeatedly called
	-- Always call Sleep() at least once in this function if it returns true
	
	if (Debug() >= DebugThreshold - 1) then print(string.format("loop %.2f", RunTime())); end;
	
	-- test the result of fetching values from the Engine

	RAM024 = RAM(0x024, Engine, TIU);	-- q1F84	-- Status Byte 2
	PacketCount = PacketCount + 2;
	if (RAM024 == nil) then
		Beep();
		print("RAM(0x024) returned nil");
		BumpCounter(COUNTER02);
		return Stop("RAM read failed");
		else
		if (Debug() >= DebugThreshold) then print("RAM 0x024 (", Check024, ") = ", RAM024); end;
		if (RAM024 ~= Check024) then
			print("RAM(0x024) Readback failed");
			BumpCounter(COUNTER01);
			end
		end
	Yield();
	
	if (Bell(OFF, 0, Engine, TIU) == nil) then	-- bFFFB
		Beep();
		BumpCounter(COUNTER02);
		return Stop("Bell Off command failed");
		end;
	PacketCount = PacketCount + 1;
	--
	-- Note 0x008 and 0x009 is the scale factor
	-- in RTC Engine, it is hard coded as 0x5485 (big endian)
	-- RAM 0x008 = 84 (dec) 0x54 and RAM 0x009 = 133 (dec) 0x85
	RAM008 = RAM(0x008, Engine, TIU);	-- q3F84	-- Scaling Factor
	PacketCount = PacketCount + 2;	
	if (RAM008 == nil) then
		Beep();
		print("RAM(0x008) returned nil");
		BumpCounter(COUNTER02);
		return Stop("RAM read failed");
		else	
		if (Debug() >= DebugThreshold) then print("RAM 0x008 (", Check008, ") = ", RAM008); end;
		if (RAM008 ~= Check008) then
			print("RAM(0x008) Readback failed");
			BumpCounter(COUNTER01);
			end
		end
		
	RAM009 = RAM(0x009, Engine, TIU);	-- q3FB3	-- Scaling Factor
	PacketCount = PacketCount + 2;	
	if (RAM009 == nil) then
		Beep();
		print("RAM(0x009) returned nil");
		BumpCounter(COUNTER02);
		return Stop("RAM read failed");
		else
		if (Debug() >= DebugThreshold) then print("RAM 0x009 (", Check009, ") = ", RAM009); end;
		if (RAM009 ~= Check009) then
			print("RAM(0x009) Readback failed");
			BumpCounter(COUNTER01);
			end
		end
	Yield();
	
	if (Whistle(OFF, 0, Engine, TIU) == nil) then	-- bFFFD
		Beep();
		BumpCounter(COUNTER02);
		return Stop("Whistle Off command failed");
		end;
	PacketCount = PacketCount + 1;
		
	RAM067 = RAM(0x067, Engine, TIU);	-- q3F84	-- Direction
	PacketCount = PacketCount + 2;	
	if (RAM067 == nil) then
		Beep();
		print("RAM(0x067) returned nil");
		BumpCounter(COUNTER02);
		return Stop("RAM read failed");
		else
		if (Debug() >= DebugThreshold) then print("RAM 0x067 (", Check067, ") = ", RAM067); end;
		if (RAM067 ~= Check067) then
			print("RAM(0x067) Readback failed");
			BumpCounter(COUNTER01);
			end
		end
	Yield();
	
	RAM009 = RAM(0x009, Engine, TIU);	-- q3F84	-- Scaling Factor
	PacketCount = PacketCount + 2;	
	if (RAM009 == nil) then
		Beep();
		print("RAM(0x009) returned nil");
		BumpCounter(COUNTER02);
		return Stop("RAM read failed");
		else
		if (Debug() >= DebugThreshold) then print("RAM 0x009 (", Check009, ") = ", RAM009); end;
		if (RAM009 ~= Check009) then
			print("RAM(0x009) Readback failed");
			BumpCounter(COUNTER01);
			end
		end
	Yield();

	BumpCounter(COUNTER03);
	Sleep(0.1);
	return true;	-- false=do not loop, true=continue to loop
	end
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
	-- the cleanup() function is called once when the [STOP] button is pressed
	print(string.format("cleanup %.2f", RunTime()));
	-- print loops per second
	print("Loops per second = ", GetCounter(COUNTER03)/GetCounter(ELAPSEDTIME));
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
