--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2019 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
--[[---------------------------------------------------------------------------------------]]
-- This test measures the stopping distance of an engine.
-- Set the starting and ending speeds and the starting and ending deceleration rates.
-- The script will run all combinations and print the results.
--
-- Creates a table constructor for insertion into the "Three Engine Demo V2.lua" script or
-- the "LayoutX.lua" scripts for each layout
-- [11] = {[1] = 0.0, [2] = 0.5}
--[[---------------------------------------------------------------------------------------]]
title = "Deceleration Test"

require("defines");

-- Counter Labels
Counter01Label = "Outer Loops";
NO_LOOPS = COUNTER01;
Counter02Label = "No of Starts";
NO_STARTS = COUNTER02;
Counter03Label = "No of Stops";
NO_STOPS = COUNTER03;
-- starting time
DTOStart = 0.0;
-- Program Variables
StartingSpeed = 1;		-- starting engine speed in Smph, normally 1
EndingSpeed = 40;		-- ending engine speed in Smph, normally 40
TerminalSpeed = 0;		-- low speed to terminate each loop, normally 0
StartingDecel = 4;		-- beginning deceleration rate, normally 1
EndingDecel = 4;		-- ending deceleration rate, normally 1
Decel = StartingDecel;
EngSpeed = StartingSpeed;
Spd = 0;
DTORunning = 0.0;
DTOStopped = 0.0;
EngineSoundSave = -1;
SoundOff = false;
-- keep the noise to a minimum
AutoBell = false;
AutoWhistle = false;

-- state machine states
IDLE = 0;
DONE = 1;
S0 = 2;
S1 = 3;
S2 = 4;
S3 = 5;
S4 = 6;
S5 = 7;
S6 = 8;
-- starting state
SM = IDLE;
LastTime = 0;
--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
--
print("setup : TIU " .. TIU .. " Engine " .. Engine .. " Debug " .. Debug());
print(string.format("Run Time = %.2f Seconds", RunTime()));
print("Check condition of traction tires before using the results of this script");
Smoke(OFF, 0, Engine, TIU);
Name, PS = GetName(Engine);
print("Engine#" .. Engine .. " " .. Name .. " PS" .. PS);
Title(title .. " - " .. GetName(Engine));
StartUp(0, Engine, TIU);	-- start selected engine
-- let engine startup
Sleep(15);
-- Stop the Garbage collector since this is a real-time program
collectgarbage("stop")
print("Setup : Garbage Collection stopped");
print("StartingSpeed  " .. StartingSpeed);
print("EndingSpeed  " .. EndingSpeed);
print("TerminalSpeed  " .. TerminalSpeed);
print("StartingDecel  " .. StartingDecel);
print("EndingDecel  " .. EndingDecel);

print(string.format("Run Time = %.2f Seconds", RunTime()));
DTOStart = DTO(Engine, TIU);
print(string.format("Trip Odometer Start = %.2f Smiles", DTOStart));
print(string.format("Run Time = %.2f Seconds", RunTime()));
print(string.format("Stopping Distance-Smph-Rate------Calc------Stopping Time"));
print(string.format("Smiles   Inches        Smph/sec  Smph/Sec    Seconds"));
TableConstructor = "[" .. Engine .. "] = {";
SM = S0;	-- start the state machine
return true;
end
--[[---------------------------------------------------------------------------------------]]
function loop(Engine, TIU)
--
--[[---------------------------------------------------------------------------------------]]
-- STATE MACHINE
--[[---------------------------------------------------------------------------------------]]	

--[[---------------------------------------------------------------------------------------]]	
if (SM == IDLE) then	-- do nothing 
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
if (SM == DONE) then	-- finish up
	print("Almost Done...");
	return Stop("DONE");
	end
if (SM == S0) then
	if (Decel > EndingDecel) then
		SM = DONE;
		return true;	-- false=stop calling loop(), true=continue to call loop()
		end
	-- set deceleration value
	Rate(1, Decel, 0, Engine, TIU);
	SM = S1;
	if(Debug() > 1) then print("S0 : Set deceleration rate to " .. Decel); end;
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
if (SM == S1) then
	if (EngSpeed > EndingSpeed) then
		SM = S6;
		return true;	-- false=stop calling loop(), true=continue to call loop()
		end 
	BumpCounter(NO_STARTS);
	Beep();
	if(Debug() > 1) then print("S1 : Set speed to " .. EngSpeed .. " Smph"); end
	Throttle(EngSpeed, 0, Engine, TIU);
	SM = S2;
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
		
if (SM == S2) then
	Spd = GetSpeed(Engine, TIU);
	if (Spd ~= nil and Spd == EngSpeed) then
		if(Debug() > 1) then print("S2 : Reached speed = " .. Spd); end
		Sleep(10);
		SM = S3;
		return true;	-- false=stop calling loop(), true=continue to call loop()
		end
	Sleep(0.5);
	-- stay in S2
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
		
if (SM == S3) then
	BumpCounter(NO_STOPS);
	Beep();
	if(Debug() > 1) then print("S3 : Set speed to " .. TerminalSpeed); end
	DTORunning = DTO(Engine, TIU);
	LastTime = MicroSeconds();
	Throttle(TerminalSpeed,0, Engine, TIU);
	SM = S4;
	if(Debug() > 1) then print(string.format("S3 : Trip Odometer @ Speed = %.5f Smiles", DTORunning)); end
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
		
if (SM == S4) then
	Spd = GetSpeed(Engine, TIU);
	if (Spd ~= nil and Spd == TerminalSpeed) then
		now = MicroSeconds();
		DTOStopped = DTO(Engine, TIU);
		if(Debug() > 1) then print(string.format("S4 : Trip Odometer Stopped = %.5f Smiles", DTOStopped)); end
		Smiles = DTOStopped - DTORunning;
		SFeet = Smiles * 5280.0;
		SInches = SFeet * 12.0;
		Inches = SInches / 48.0;
		-- d = 1/2 a (t^2) from Physics I
		-- a = 2 * d /( t * t)
		delta = (now - LastTime)/ 1000000.0;
		CalcDecel = 2.0 * Smiles * 3600.0 / (delta * delta);	-- Smiles/hr/sec
		print(string.format("%7.5f %5.1f     %2d   %2d        %4.1f        %6.3f", Smiles, Inches, EngSpeed, Decel, CalcDecel, delta));
		TableConstructor = TableConstructor .. "[" .. EngSpeed .. "] = " .. Inches .. ", ";
		Sleep(5);
		SM = S5;
		return true;	-- false=stop calling loop(), true=continue to call loop()
		end
	-- for highest accuracy, check the speed as quickly as possible
	-- stay in S4
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
	
if (SM == S5) then
	EngSpeed = EngSpeed + 1;
	if(Debug() > 1) then print("S5 : New Engine Speed = " .. EngSpeed); end
	SM = S1;
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
		
if (SM == S6) then
	Decel = Decel + 1;
	EngSpeed = StartingSpeed;
	BumpCounter(NO_LOOPS);
	print(" ");
	if(Debug() > 1) then print("S6 : New deceleration rate = " .. Decel); end
	SM = S0;
	return true;	-- false=stop calling loop(), true=continue to call loop()
	end
print("Invalid State " .. SM);		
return false;	-- false=stop calling loop(), true=continue to call loop()
end
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
--
print("cleanup : TIU " .. TIU .. " Engine " .. Engine .. " Debug " .. Debug());

SetSpeed(0, 0, Engine, TIU);
DTOFinish = DTO(Engine, TIU);
-- wait for dispatch list to empty
while(CountDispatchList() > 0) do Sleep(1); end
print(string.format("Run Time = %.2f Seconds", RunTime()));
print(string.format("Trip Odometer Finish = %.2f Smiles", DTOFinish));
-- reset acceleration and deceleration values
Rate(1, 1, 0, Engine, TIU);
ShutDown(0, Engine, TIU);
if (SoundOff) then
	SoundOnButton(Engine, TIU)
	end;
if (TableConstructor ~= nil) then
	TableConstructor = string.sub(TableConstructor, 1, -3) .. "}";	-- remove ending comma
	print (TableConstructor);
	end;
return true;
end
--[[---------------------------------------------------------------------------------------]]
function PTC(Eng, TIU)
	print(string.format("Run Time = %.2f Seconds", RunTime()));
	print(TableConstructor);
	return true;
end
Function01Name, Function01Label = PTC, "Table Constructor";
--[[---------------------------------------------------------------------------------------]]
function ShowDTO(Eng, TIU)
	print(string.format("Run Time = %.2f Seconds", RunTime()));
	print(string.format("Trip Odometer = %.2f Smiles", DTO(Eng, TIU)));
	return true;
end
Function10Name, Function10Label = ShowDTO, "Trip Odometer";
--[[---------------------------------------------------------------------------------------]]

--[[---------------------------------------------------------------------------------------]]
function SoundOffButton(Eng, TIU)
	print("Sound Off Button()");
	S1 = GetVolume(MASTER_VOLUME, Eng, TIU);	-- get Volume
	print(" Sound Off - Engine " .. Eng .. " Level was " .. S1);
	EngineSoundSave = S1;
	SoundOff = true;
	SetVolume(MASTER_VOLUME, 0, 0, Eng, TIU);
	return true;
end
Function04Name, Function04Label = SoundOffButton, "Sound Off";
--[[---------------------------------------------------------------------------------------]]
function SoundOnButton(Eng, TIU)
	print("Sound On Button()");
	if (SoundOff) then
		S1 = EngineSoundSave;		
		print(" Sound On - Engine " .. Eng .. " Level restored to " .. S1);
		SoundOff = false;;
		SetVolume(MASTER_VOLUME, S1, 0, Eng, TIU);
		end;
	return true;
end
Function05Name, Function05Label = SoundOnButton, "Sound On";
--[[---------------------------------------------------------------------------------------]]
--[[---------------------------------------------------------------------------
Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 13 Debug 0
Run Time = 5.80 Seconds
Check condition of traction tires before using the results of this script
Engine#13 P&LE GP7 #1500   PS3
Run Time = 20.80 Seconds
Trip Odometer Start = 0.00 Smiles
Run Time = 21.60 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00012   0.2      1    1         3.8         0.480
0.00038   0.5      2    1         6.5         0.650
0.00081   1.1      3    1         8.0         0.850
0.00122   1.6      4    1         5.7         1.240
0.00143   1.9      5    1         6.5         1.260
0.00225   3.0      6    1         6.2         1.620
0.00291   3.8      7    1         5.0         2.057
0.00371   4.9      8    1         5.6         2.187
0.00444   5.9      9    1         5.5         2.405
0.00528   7.0     10    1         5.0         2.769
0.00613   8.1     11    1         5.1         2.956
0.00723   9.5     12    1         4.9         3.246
0.00828  10.9     13    1         4.9         3.505
0.00952  12.6     14    1         4.9         3.725
0.01075  14.2     15    1         4.8         4.023
0.01201  15.9     16    1         4.6         4.322
0.01332  17.6     17    1         4.7         4.502
0.01471  19.4     18    1         4.7         4.722
0.01616  21.3     19    1         4.7         4.987
0.01754  23.1     20    1         4.7         5.175
0.01810  23.9     21    1         4.5         5.390
0.02087  27.5     22    1         4.5         5.810
0.02288  30.2     23    1         4.6         5.971
0.02476  32.7     24    1         4.3         6.421
0.02696  35.6     25    1         4.6         6.481
0.02884  38.1     26    1         4.5         6.786
0.03055  40.3     27    1         4.4         7.110
0.03332  44.0     28    1         4.4         7.401
0.03499  46.2     29    1         4.4         7.569
0.03581  47.3     30    1         4.5         7.542
0.03954  52.2     31    1         4.4         8.040
0.04205  55.5     32    1         4.4         8.313
0.04461  58.9     33    1         4.3         8.668
0.04533  59.8     34    1         4.3         8.666
0.05009  66.1     35    1         4.3         9.114
0.05258  69.4     36    1         4.3         9.392
0.05546  73.2     37    1         4.3         9.676
0.05792  76.5     38    1         4.2         9.950
0.06105  80.6     39    1         4.2        10.207
0.06352  83.8     40    1         4.3        10.330
 
Almost Done...
DONE
cleanup : TIU 1 Engine 13 Debug 0
Run Time = 1113.10 Seconds
Trip Odometer Finish = 4.07 Smiles
[13] = {[1] = 0.16055363321799, [2] = 0.5038062283737, [3] = 1.0629757785467, [4] = 1.6166089965398, [5] = 1.8934256055363, [6] = 2.9674740484429, [7] = 3.8477508650519, [8] = 4.8941176470588, [9] = 5.8574394463668, [10] = 6.9647058823529, [11] = 8.0885813148789, [12] = 9.5391003460208, [13] = 10.928719723183, [14] = 12.561937716263, [15] = 14.195155709343, [16] = 15.856055363322, [17] = 17.57785467128, [18] = 19.415916955017, [19] = 21.331487889273, [20] = 23.147404844291, [21] = 23.889273356401, [22] = 27.543252595156, [23] = 30.200692041522, [24] = 32.680968858131, [25] = 35.593079584775, [26] = 38.062283737024, [27] = 40.321107266436, [28] = 43.98615916955, [29] = 46.184083044983, [30] = 47.263667820069, [31] = 52.196539792388, [32] = 55.507266435986, [33] = 58.884429065744, [34] = 59.836678200692, [35] = 66.120415224914, [36] = 69.408996539792, [37] = 73.212456747405, [38] = 76.456747404844, [39] = 80.586851211072, [40] = 83.847750865053}


Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 16 Debug 0
Run Time = 1110.60 Seconds
Check condition of traction tires before using the results of this script
Engine#16 P&LE RS-3 #8356  PS3
Run Time = 1125.60 Seconds
Trip Odometer Start = 3.37 Smiles
Run Time = 1126.10 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00002   0.0      1    1         1.7         0.265
0.00036   0.5      2    1        10.6         0.492
0.00077   1.0      3    1         6.6         0.913
0.00115   1.5      4    1         6.9         1.095
0.00146   1.9      5    1         6.4         1.284
0.00196   2.6      6    1         6.4         1.486
0.00291   3.8      7    1         5.0         2.042
0.00316   4.2      8    1         5.2         2.101
0.00441   5.8      9    1         5.0         2.529
0.00533   7.0     10    1         5.1         2.737
0.00621   8.2     11    1         5.0         2.975
0.00715   9.4     12    1         4.7         3.308
0.00838  11.1     13    1         4.8         3.539
0.00934  12.3     14    1         4.9         3.715
0.01064  14.0     15    1         4.6         4.097
0.01196  15.8     16    1         4.6         4.307
0.01338  17.7     17    1         4.7         4.505
0.01473  19.5     18    1         4.6         4.799
0.01517  20.0     19    1         4.8         4.767
0.01760  23.2     20    1         4.5         5.303
0.01821  24.0     21    1         4.7         5.282
0.02113  27.9     22    1         4.4         5.864
0.02281  30.1     23    1         4.4         6.135
0.02556  33.7     24    1         4.4         6.493
0.02638  34.8     25    1         3.7         7.126
0.02896  38.2     26    1         4.6         6.765
0.03072  40.6     27    1         4.5         7.035
0.03138  41.4     28    1         4.3         7.233
0.03509  46.3     29    1         4.4         7.585
0.03720  49.1     30    1         4.3         7.885
0.03954  52.2     31    1         4.3         8.120
0.04036  53.3     32    1         4.3         8.222
0.04481  59.2     33    1         4.3         8.686
0.04728  62.4     34    1         4.4         8.845
0.04815  63.6     35    1         4.2         9.119
0.05052  66.7     36    1         4.3         9.227
0.05535  73.1     37    1         4.2         9.710
0.05810  76.7     38    1         4.3         9.887
0.05882  77.6     39    1         4.3         9.882
0.06186  81.6     40    1         4.3        10.152
 
Almost Done...
DONE
cleanup : TIU 1 Engine 16 Debug 0
Run Time = 2220.20 Seconds
Trip Odometer Finish = 7.46 Smiles
[16] = {[1] = 0.022184221472248, [2] = 0.4714147062904, [3] = 1.014928132366, [4] = 1.5140731154966, [5] = 1.9300272681061, [6] = 2.5844618015441, [7] = 3.8434163701067, [8] = 4.170633636826, [9] = 5.8178120811572, [10] = 7.0323982067753, [11] = 8.1915237787124, [12] = 9.4393862365397, [13] = 11.064380459398, [14] = 12.323335027962, [15] = 14.048158247447, [16] = 15.789619633036, [17] = 17.66418634746, [18] = 19.450016175995, [19] = 20.026805934279, [20] = 23.232425937052, [21] = 24.042150020798, [22] = 27.885566390905, [23] = 30.115080648887, [24] = 33.736654804271, [25] = 34.823681656422, [26] = 38.228959652447, [27] = 40.552756851689, [28] = 41.417941489115, [29] = 46.320654434533, [30] = 49.099228173961, [31] = 52.193927069371, [32] = 53.280953921524, [33] = 59.154226556362, [34] = 62.415307112816, [35] = 63.563340574016, [36] = 66.691315801636, [37] = 73.058187364237, [38] = 76.690853630356, [39] = 77.644775153672, [40] = 81.649027129454}


Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 17 Debug 0
Run Time = 551.60 Seconds
Check condition of traction tires before using the results of this script
Engine#17 P&LE MP15AD#1594 PS3
Run Time = 566.60 Seconds
Trip Odometer Start = 2.85 Smiles
Run Time = 567.10 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00013   0.2      1    1         4.3         0.467
0.00037   0.5      2    1         6.5         0.645
0.00082   1.1      3    1         7.6         0.879
0.00123   1.6      4    1         5.3         1.290
0.00169   2.2      5    1         5.8         1.453
0.00226   3.0      6    1         5.9         1.656
0.00298   3.9      7    1         5.0         2.061
[STOP]
cleanup : TIU 1 Engine 17 Debug 0
Run Time = 699.70 Seconds
Trip Odometer Finish = 2.94 Smiles
[17] = {[1] = 0.17192771641124, [2] = 0.49359892776264, [3] = 1.0759347414155, [4] = 1.624994222859, [5] = 2.2295142579841, [6] = 2.9782317326802, [7] = 3.9321532559962}

Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 17 Debug 0
Run Time = 809.10 Seconds
Check condition of traction tires before using the results of this script
Engine#17 P&LE MP15AD#1594 PS3
Run Time = 824.10 Seconds
Trip Odometer Start = 3.27 Smiles
Run Time = 824.60 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00017   0.2      1    1         4.9         0.498
0.00044   0.6      2    1         7.8         0.640
0.00081   1.1      3    1         7.3         0.894
0.00123   1.6      4    1         5.7         1.249
0.00168   2.2      5    1         5.7         1.452
0.00224   3.0      6    1         4.9         1.823
0.00289   3.8      7    1         0.0        36.264    <<<--- OOPS
0.00361   4.8      8    1         5.1         2.260
[STOP]
cleanup : TIU 1 Engine 17 Debug 0
Run Time = 1011.70 Seconds
Trip Odometer Finish = 3.39 Smiles
[17] = {[1] = 0.22184221472482, [2] = 0.58233581365222, [3] = 1.0703886860475, [4] = 1.624994222859, [5] = 2.2184221472476, [6] = 2.9560475112079, [7] = 3.810140037898, [8] = 4.7696076165831}

Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 17 Debug 0
Run Time = 1022.40 Seconds
Check condition of traction tires before using the results of this script
Engine#17 P&LE MP15AD#1594 PS3
Run Time = 1037.40 Seconds
Trip Odometer Start = 3.39 Smiles
Run Time = 1037.90 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00024   0.3      1    1         7.7         0.477
0.00044   0.6      2    1         7.0         0.670
0.00074   1.0      3    1         7.1         0.864
0.00122   1.6      4    1         5.6         1.249
0.00170   2.2      5    1         6.1         1.412
0.00232   3.1      6    1         6.2         1.646
0.00289   3.8      7    1         4.8         2.082
0.00363   4.8      8    1         5.4         2.199
0.00444   5.9      9    1         5.2         2.480
0.00516   6.8     10    1         4.7         2.813
0.00618   8.2     11    1         4.9         3.016
0.00737   9.7     12    1         5.0         3.265
0.00838  11.1     13    1         4.9         3.511
0.00957  12.6     14    1         4.8         3.805
0.01072  14.1     15    1         4.6         4.105
0.01187  15.7     16    1         4.9         4.165
0.01329  17.5     17    1         4.7         4.535
0.01464  19.3     18    1         4.7         4.718
0.01601  21.1     19    1         4.5         5.089
0.01771  23.4     20    1         4.5         5.296
0.01931  25.5     21    1         4.6         5.479
0.02097  27.7     22    1         4.4         5.860
0.02289  30.2     23    1         4.4         6.095
0.02479  32.7     24    1         4.5         6.287
0.02663  35.2     25    1         4.6         6.477
0.02867  37.8     26    1         4.4         6.851
0.03074  40.6     27    1         4.4         7.091
0.03291  43.4     28    1         4.3         7.409
0.03486  46.0     29    1         4.4         7.583
0.03745  49.4     30    1         4.4         7.828
0.03991  52.7     31    1         4.4         8.037
0.04212  55.6     32    1         4.3         8.427
0.04449  58.7     33    1         4.4         8.541
0.04701  62.1     34    1         4.4         8.751
0.04975  65.7     35    1         4.3         9.134
0.05260  69.4     36    1         4.4         9.328
0.05525  72.9     37    1         4.2         9.703
0.05820  76.8     38    1         4.3         9.891
0.06083  80.3     39    1         4.3        10.104
0.06417  84.7     40    1         4.3        10.368
 
Almost Done...
DONE
cleanup : TIU 1 Engine 17 Debug 0
Run Time = 2132.00 Seconds
Trip Odometer Finish = 7.49 Smiles
[17] = {[1] = 0.32167121135082, [2] = 0.57678975828431, [3] = 0.97055968942096, [4] = 1.6083560567547, [5] = 2.2461524240884, [6] = 3.0614225632019, [7] = 3.8212321486344, [8] = 4.7973378934233, [9] = 5.8566344687338, [10] = 6.8050099366825, [11] = 8.1527013911352, [12] = 9.7277811156818, [13] = 11.064380459398, [14] = 12.633914128576, [15] = 14.147987244073, [16] = 15.662060359569, [17] = 17.542173129362, [18] = 19.322456902528, [19] = 21.136017007903, [20] = 23.371077321255, [21] = 25.489670471877, [22] = 27.680362342285, [23] = 30.214909645515, [24] = 32.727272727273, [25] = 35.156444978509, [26] = 37.846281832048, [27] = 40.580487128531, [28] = 43.447797753847, [29] = 46.015621389287, [30] = 49.431991496048, [31] = 52.687525997134, [32] = 55.593659010029, [33] = 58.732726348385, [34] = 62.054813513888, [35] = 65.670841613903, [36] = 69.431067153488, [37] = 72.925082035402, [38] = 76.82395895919, [39] = 80.301335675001, [40] = 84.699357581919}


Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 2 Debug 0
Run Time = 4.20 Seconds
Check condition of traction tires before using the results of this script
Engine#2 A&S 0-8-0 #211   PS2
Setup : Garbage Collection stopped
StartingSpeed  1
EndingSpeed  40
TerminalSpeed  0
StartingDecel  4
EndingDecel  4
Run Time = 19.20 Seconds
Trip Odometer Start = 2.08 Smiles
Run Time = 20.00 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00013   0.2      1    4         2.1         0.670
0.00032   0.4      2    4         5.0         0.675
0.00060   0.8      3    4         9.3         0.681
0.00079   1.0      4    4        12.0         0.691
0.00103   1.4      5    4         7.8         0.975
0.00126   1.7      6    4         9.1         0.995
0.00160   2.1      7    4        11.6         0.995
0.00189   2.5      8    4        14.3         0.975
0.00237   3.1      9    4        10.6         1.270
0.00256   3.4     10    4        11.6         1.260
0.00313   4.1     11    4        13.2         1.305
0.00347   4.6     12    4        15.8         1.256
0.00385   5.1     13    4        17.3         1.267
0.00430   5.7     14    4        19.2         1.270
0.00472   6.2     15    4        14.0         1.556
0.00499   6.6     16    4        15.8         1.510
0.00556   7.3     17    4        16.3         1.569
0.00621   8.2     18    4        17.6         1.595
0.00675   8.9     19    4        19.5         1.578
0.00734   9.7     20    4        14.8         1.890
0.00776  10.2     21    4        15.2         1.920
0.00827  10.9     22    4        18.7         1.784
0.00924  12.2     23    4        14.9         2.116
0.00940  12.4     24    4        14.9         2.129
0.01031  13.6     25    4        15.5         2.186
0.01072  14.1     26    4        16.1         2.188
0.01138  15.0     27    4        17.3         2.178
0.01198  15.8     28    4        14.9         2.408
0.01282  16.9     29    4        15.6         2.433
0.01347  17.8     30    4        17.3         2.368
0.01472  19.4     31    4        17.3         2.477
0.01471  19.4     32    4        17.4         2.466
0.01575  20.8     33    4        17.8         2.523
0.01667  22.0     34    4        16.0         2.742
0.01778  23.5     35    4        16.5         2.785
0.01849  24.4     36    4        17.3         2.776
0.01917  25.3     37    4        17.9         2.777
0.02028  26.8     38    4        16.1         3.010
0.02099  27.7     39    4        16.2         3.051
0.02151  28.4     40    4        17.0         3.020
 
Almost Done...
DONE
cleanup : TIU 1 Engine 2 Debug 0
Run Time = 976.70 Seconds
Trip Odometer Finish = 5.62 Smiles
[2] = {[1] = 0.17567341475633, [2] = 0.41659695499419, [3] = 0.78802074619334, [4] = 1.0490212481183, [5] = 1.360214154258, [6] = 1.6613685795547, [7] = 2.1131002175004, [8] = 2.4995817299651, [9] = 3.1320060230886, [10] = 3.377948803748, [11] = 4.1358541074113, [12] = 4.5825665049355, [13] = 5.0844905470977, [14] = 5.6817801572691, [15] = 6.2288773632258, [16] = 6.5852434331605, [17] = 7.3431487368244, [18] = 8.2014388489208, [19] = 8.9041325079473, [20] = 9.6921532541406, [21] = 10.24928894094, [22] = 10.911828676594, [23] = 12.196754224528, [24] = 12.412581562657, [25] = 13.607160783002, [26] = 14.149238748536, [27] = 15.027605822319, [28] = 15.815626568512, [29] = 16.92487870169, [30] = 17.778149573364, [31] = 19.429479672076, [32] = 19.419441191234, [33] = 20.789693826334, [34] = 22.009369248787, [35] = 23.464948971056, [36] = 24.403546929899, [37] = 25.301990965367, [38] = 26.772628408901, [39] = 27.701187886899, [40] = 28.398862305504}
Success compiling D:\trains\MTH\PC Control\Scripts\Deceleration Test.lua
setup : TIU 1 Engine 2 Debug 0
Run Time = 1099.30 Seconds
Check condition of traction tires before using the results of this script
Engine#2 A&S 0-8-0 #211   PS2
Setup : Garbage Collection stopped
StartingSpeed  1
EndingSpeed  40
TerminalSpeed  0
StartingDecel  1
EndingDecel  1
Run Time = 1114.30 Seconds
Trip Odometer Start = 5.62 Smiles
Run Time = 1114.80 Seconds
Stopping Distance-Smph-Rate------Calc------Stopping Time
Smiles   Inches        Smph/sec  Smph/Sec    Seconds
0.00021   0.3      1    1         3.5         0.660
0.00042   0.6      2    1         3.2         0.975
0.00076   1.0      3    1         5.8         0.975
0.00125   1.7      4    1         5.2         1.311
0.00181   2.4      5    1         5.5         1.544
0.00240   3.2      6    1         4.8         1.900
0.00302   4.0      7    1         4.7         2.151
0.00378   5.0      8    1         4.5         2.446
0.00451   5.9      9    1         4.3         2.738
0.00533   7.0     10    1         5.0         2.769
0.00643   8.5     11    1         5.0         3.032
0.00731   9.6     12    1         4.6         3.365
0.00848  11.2     13    1         4.7         3.620
0.00954  12.6     14    1         4.4         3.952
0.01075  14.2     15    1         4.7         4.047
0.01226  16.2     16    1         4.3         4.531
0.01335  17.6     17    1         4.6         4.547
0.01483  19.6     18    1         4.6         4.818
0.01648  21.8     19    1         4.4         5.189
0.01790  23.6     20    1         4.6         5.301
0.01967  26.0     21    1         4.3         5.712
0.02126  28.1     22    1         4.3         5.973
0.02319  30.6     23    1         4.2         6.339
0.02504  33.1     24    1         4.5         6.332
0.02707  35.7     25    1         4.5         6.588
0.02899  38.3     26    1         4.4         6.880
0.03114  41.1     27    1         4.3         7.219
0.03325  43.9     28    1         4.2         7.519
0.03552  46.9     29    1         4.4         7.619
0.03758  49.6     30    1         4.2         8.019
0.04032  53.2     31    1         4.3         8.196
0.04259  56.2     32    1         4.1         8.642
0.04514  59.6     33    1         4.3         8.670
0.04790  63.2     34    1         4.2         9.016
0.04988  65.8     35    1         4.3         9.157
0.05268  69.5     36    1         4.2         9.549
0.05564  73.4     37    1         4.2         9.788
0.05851  77.2     38    1         4.3         9.926
0.06152  81.2     39    1         4.2        10.222
0.06430  84.9     40    1         4.2        10.555
 
Almost Done...
DONE
cleanup : TIU 1 Engine 2 Debug 0
Run Time = 2219.20 Seconds
Trip Odometer Finish = 9.75 Smiles
[2] = {[1] = 0.28107746360966, [2] = 0.55211644637755, [3] = 1.0088673247447, [4] = 1.6513300987123, [5] = 2.3841392002677, [6] = 3.1621214656175, [7] = 3.9902961351844, [8] = 4.9891249790862, [9] = 5.9477998996149, [10] = 7.0319558306841, [11] = 8.4825163125316, [12] = 9.6469800903456, [13] = 11.197925380626, [14] = 12.593274217835, [15] = 14.18939267191, [16] = 16.187050359712, [17] = 17.617533879874, [18] = 19.580056884725, [19] = 21.758407227706, [20] = 23.625564664548, [21] = 25.959511460599, [22] = 28.062573197256, [23] = 30.612347331437, [24] = 33.051698176342, [25] = 35.726953321064, [26] = 38.266688974402, [27] = 41.102559812615, [28] = 43.888238246613, [29] = 46.889744018738, [30] = 49.610172327254, [31] = 53.219006190396, [32] = 56.220511962523, [33] = 59.583403045005, [34] = 63.232390831521, [35] = 65.847415091182, [36] = 69.531537560649, [37] = 73.44654508951, [38] = 77.231052367408, [39] = 81.201271540907, [40] = 84.870336289107}

---------------------------------------------------------------------------]]
