--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2022 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
local MyDirection;
title = "RFID Sensor Test";
--
-- This sketches callc complied c++ code that interfaces with components contained in the Internet 
-- Suite by Francois Piette (ICS) v5r20 for BCB4.0/6.0 code and v8.58 for CB10.1.
--
require([[defines]]);
require([[functions]]);
--
--[[
When using this module, all variables (including functions!) must be declared through 
a regular assignment (even assigning nil will do) in a strict scope before being used 
anywhere or assigned to inside a nested scope.
https://github.com/lua-stdlib/strict
]]
local _ENV = require 'std.strict' (_G)	-- strict.lua will detect undeclared global variables
local _;
--
local TagCount = 0;
local EngineFrontTagCount = 0;
-- Counter Labels
Counter01Label = "Tags Count";
Counter03Label = "Speed Limit";
local C3 = COUNTER03;
Counter04Label = "Engine Count";
local C4 = COUNTER04;
Counter05Label = "Car Count";
local C5 = COUNTER05;
Counter06Label = "Cab/Obs Count";
local C6 = COUNTER06;

local Tags = {};		-- table to hold all of the tags detected in sequence
local SoundOn = true;
--local StartBlockNo, EndBlockNo;
local CurrentBlockNo;
--local LastBlockNo, NextBlockNo;
local StartedFlag = false;
local RFIDFlag = false;
local Headway = false;
local Layout4;
local MyTIUNo, Enum = 0, 0;
local MyEngineNo = 0;
local SoundOff = false;
local EngineSoundSave;
local BeepOn = false;
local MySpeed = 25;

-- These labels appear on each of the tabs for the Function Buttons
Tab01Label = "Control";
Tab02Label = "Support";
Tab03Label = "Not Used";
--[[---------------------------------------------------------------------------------------]]

--[[---------------------------------------------------------------------------------------]]

--[[---------------------------------------------------------------------------------------]]

function setup(Engine, TIU)
print("setup() Running");

-- Instrumented Layout 4
Layout4 = require([[Layout4]]);		-- All of the details about the Layout
-- an undirected weighted double vertex graph
MyTIUNo = Layout4.MASTER_TIU;
-- Note: this script will work either with or without the Layout Window (run "Layout4 APB Master.lua" to get the layout)

MyEngineNo = Engine;
Title(title, MyEngineNo);
MyDirection = WB;

math.randomseed(os.time());	-- random seed

for iEng = 1, 99 do	-- check all possible engines on one TIU
	if (Exists(iEng)) then
		local Name, PS;
		Name, PS = GetName(iEng);
		print(iEng .. " " .. Name .. " PS" .. PS);
		end
	end
	
Enum = tonumber(InputBox(title, "Enter Engine Number", MyEngineNo));
if (Enum ~= nil and Enum ~= 0) then	-- if the result is a number, then use it, otherwise, ignore it
	MyEngineNo = Enum;
	else
	return Stop("Canceled");
	end
print("Setup : Engine Number = " .. MyEngineNo);
-- Show new title with Engine # and Name and put Engine # into Engine Number Spinner
local Name = GetName(MyEngineNo);
Title(title .. " #" .. MyEngineNo .. " " .. Name, MyEngineNo);
--]]	
if (Layout4.StoppingisEmpty(MyEngineNo,1) == nil) then
	return Stop("No StoppingDistance Table for Engine " .. MyEngineNo);
	end	
--]]
--
-- start of run
--
-- must ask the dispatcher for direction of travel
	local Enum = tonumber(InputBox(title, "Enter Direction " .. WB .. "=" .. DirectionNames[WB] .. " or " .. EB .. "=" .. DirectionNames[EB], MyDirection));
	if (Enum ~= nil and Enum ~= 0) then	-- if the result is a number, 1 or 2, then use it, otherwise, ignore it
		if (Enum == WB or Enum == EB) then
			MyDirection = Enum;
			else
			return Stop("Canceled");
			end;
		else
		return Stop("Canceled");
		end;
	print("Setup : Direction = " .. DirectionNames[MyDirection]);

-- must ask the dispatcher for starting block number
print("Select Starting Block Number");
for k, v in pairs(BlockName) do
	if (k <= 100) then
		print(v .. "(" .. k .. ")");
		end;
	end;
--
-- read Starting Block Number
--
local response = InputQuery(title, "Enter Starting Block Number (0 for none)", ALIQUIPPA_BLOCK);
if (response == false) then
	return Stop("Canceled");
	end;
local Enum = tonumber(response);
if (Enum ~= nil and Enum ~= 0) then	-- if the result is a block number, then use it, otherwise, ignore it
	CurrentBlockNo = Enum;
	print("Setup : Starting Block = " .. BlockName[Enum]);
	end
--
-----------------------------------------------------------------------------------------------------

-- create and register the cab signals
if (LayoutShowing()) then
	print("Setup : Creating Cab Signals");
	-- Register the new block's home and distant signals
	local NumLightsHome, NumLightsDistant;
	local HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
	_, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
	local DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
	_, NumLightsDistant = GetAspect(DistantSignal);
	CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
	print("Setup : Registering the Cab Signals #", HomeSignal, " and #", DistantSignal);
	RegisterCabSignals(HomeSignal, DistantSignal);
							-- associate the Home cab signal with next signal and
							-- the Distant cab signal with second signal.
							-- Signals must be created by the "Layout4 Signal Master.lua"
							-- script which should be run in a different PC window
	print("Setup : Reflecting signals from Layout window into the Cab Display");
	print ("Setup : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[(GetCabAspect(HOMESIGNAL))]);
	print ("Setup : Distant Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[(GetCabAspect(DISTANTSIGNAL))]);
end;
-----------------------------------------------------------------------------------------------------
SetCounter(C3, MySpeed);
--[[-------------------------]]
-- Set Loop Switches to looping
Switch(REVERSE, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0)	

if (MyDirection == EB) then
	print("Setup : Eastbound");
	print("All passing/station track switches to REVERSE");
	Switch(REVERSE, Switches[ALIQUIPPA_WEST][CHAN_NUMBER], Switches[ALIQUIPPA_WEST][AIU_NUMBER], Switches[ALIQUIPPA_WEST][TIU_NUMBER], 0)	
	Switch(REVERSE, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0)
	Switch(REVERSE, Switches[COLLEGE_WEST][CHAN_NUMBER], Switches[COLLEGE_WEST][AIU_NUMBER], Switches[COLLEGE_WEST][TIU_NUMBER], 0)
	Switch(REVERSE, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	Switch(REVERSE, Switches[STRUTHERS_WEST][CHAN_NUMBER], Switches[STRUTHERS_WEST][AIU_NUMBER], Switches[STRUTHERS_WEST][TIU_NUMBER], 0)
	Switch(REVERSE, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0)
	Sleep(3);	-- time for all of the switches to align
	else
	print("Setup : Westbound");
	print("All passing/station track switches to NORMAL");
	Switch(NORMAL, Switches[ALIQUIPPA_WEST][CHAN_NUMBER], Switches[ALIQUIPPA_WEST][AIU_NUMBER], Switches[ALIQUIPPA_WEST][TIU_NUMBER], 0)	
	Switch(NORMAL, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[COLLEGE_WEST][CHAN_NUMBER], Switches[COLLEGE_WEST][AIU_NUMBER], Switches[COLLEGE_WEST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[STRUTHERS_WEST][CHAN_NUMBER], Switches[STRUTHERS_WEST][AIU_NUMBER], Switches[STRUTHERS_WEST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0)
	Sleep(3);	-- time for all of the switches to align
	end
--[[-------------------------]]
-- read up current sound volume
EngineSoundSave = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
--
return true;		-- false=setup failed, true=setup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
-- This function is called each time a tag programmed with 32 bytes of data is detected (TAGDATABLOCK4)
-- TagPacket contains 8 characters (4 hex digits) of UID and 32 characters
-- of block 4 data from the programmed tag.

BumpCounter(COUNTER01);	
local PacketLen = #TagPacket;

if (Debug() >= 8) then print("Packet Length = ", PacketLen); end;
if (Debug() >= 7) then print(TagPacket); end;

-- The complete packet received is in TagPacket 
-- (all of the fields have already been extracted into the first parameters to tag()
--   First 8 digits are the 4 byte Tag UID in hexidecimal
--   Next 32 digits are the 16 bytes of Block 4 Information read from the Tag in hexidecimal
if (Debug() >= 6) then 
	print(string.format("Packet  %s  %s", string.sub(TagPacket, 1, 8),
	string.sub(TagPacket,9,40) ));
	end

if (not RFIDFlag) then	-- ignore tags unless engine is moving
	return true;
	end;
	
-- index the table Tags[] by tag ID, value is the tag packet itself
Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
TagCount = TagCount + 1;	-- count the number of tags

if (EngineNo > 0) then	-- it's an engine
	if (EngineNo == MyEngineNo) then
		if (BeepOn) then Beep(ASTERISK); end;
		-- next line shows a different way to quote a string using brackets
		printc(clGreen, string.format([[tag(%.2f) : Detector %d Reader %d Eng#%d %s %s #%s %s]],
			RunTime(), Detector, Reader, EngineNo, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber, GetTagLocation(TagLocation)));
		if (TagLocation == TAG_ON_FRONT_TRUCK) then
			EngineFrontTagCount = EngineFrontTagCount + 1;
			print("Engine Count = " .. EngineFrontTagCount);
			BumpCounter(C4);
			--if (((Detector == 2 or Detector == 4 or Detector == 6 or Detector == 5) and Reader == 1) or ((Detector == 1 or Detector == 4 or Detector == 2 or Detector == 6 or Detector == 9 or Detector == 4) and Reader == 0))then
			--	if (((TagCount-1) % 5) ~= 0) then
			--		return Stop("Complete set of 5 tags not detected by a tag reader");
			--		else
			--		print("Tag Count OK");
			--		end;
			--	end;
			-- determine the new block
			if (MyDirection == WB) then
				-- Tag 3-0
				if (Detector == 3 and Reader == 0) then
					CurrentBlockNo = MCKEES_ROCKS_BLOCK;
				-- Tag 5-1
				elseif (Detector == 5 and Reader == 1) then
					CurrentBlockNo = J_L_THRU_TRACK;
				-- Tag 1-0
				elseif (Detector == 1 and Reader == 0) then
					CurrentBlockNo = ALIQUIPPA_BLOCK;
				-- Tag 6-0
				elseif (Detector == 6 and Reader == 0) then
					CurrentBlockNo = MONACA_BLOCK;
					-- entering tunnel, reduce volume
					if (SoundOff == false) then
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 0, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 1, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 1/2), 2, MyEngineNo, MyTIUNo);
						end;
		
				-- Tag 6-1
				elseif (Detector == 6 and Reader == 1) then
					CurrentBlockNo = BEAVER_FALLS_BLOCK;	
					-- leaving tunnel, increase volume
					if (SoundOff == false) then
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 0, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 1, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, EngineSoundSave, 2, MyEngineNo, MyTIUNo);
						end;
		
				-- Tag 4-1
				elseif (Detector == 4 and Reader == 1) then
					CurrentBlockNo = COLLEGE_THRU_TRACK;
				-- Tag 2-1
				elseif (Detector == 2 and Reader == 1) then
					CurrentBlockNo = NEW_CASTLE_BLOCK;
				-- Tag 9-0
				elseif (Detector == 9 and Reader == 0) then
					CurrentBlockNo = LOWELLVILLE_BLOCK;					
				-- Tag 2-0
				elseif (Detector == 2 and Reader == 0) then
					CurrentBlockNo = STRUTHERS_THRU_TRACK;
				-- Tag 4-0
				elseif (Detector == 4 and Reader == 0) then
					CurrentBlockNo = YOUNGSTOWN_BLOCK;
				-- Tag 5-0
				elseif (Detector == 5 and Reader == 0) then
					CurrentBlockNo = LIONEL_IVES_BLOCK;
					end;
				end;

			if (MyDirection == EB) then
				-- Tag 3-0
				if (Detector == 3 and Reader == 0) then
					CurrentBlockNo = LIONEL_IVES_BLOCK;
				-- Tag 5-1
				elseif (Detector == 5 and Reader == 1) then
					CurrentBlockNo = MCKEES_ROCKS_BLOCK;
				-- Tag 1-0
				elseif (Detector == 1 and Reader == 0) then
					CurrentBlockNo = J_L_THRU_TRACK;
				-- Tag 6-0
				elseif (Detector == 6 and Reader == 0) then
					CurrentBlockNo = ALIQUIPPA_BLOCK;
					-- leaving tunnel, increase volume
					if (SoundOff == false) then
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 0, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 1, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, EngineSoundSave, 2, MyEngineNo, MyTIUNo);
						end;
		
				-- Tag 6-1
				elseif (Detector == 6 and Reader == 1) then
					CurrentBlockNo = MONACA_BLOCK;				
					-- entering tunnel, reduce volume
					if (SoundOff == false) then
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 0, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 1, MyEngineNo, MyTIUNo);
						SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 1/2), 2, MyEngineNo, MyTIUNo);
						end;
		
				-- Tag 4-1
				elseif (Detector == 4 and Reader == 1) then
					CurrentBlockNo = BEAVER_FALLS_BLOCK;
				-- Tag 2-1
				elseif (Detector == 2 and Reader == 1) then
					CurrentBlockNo = COLLEGE_THRU_TRACK;
				-- Tag 2-0
				elseif (Detector == 2 and Reader == 0) then
					CurrentBlockNo = LOWELLVILLE_BLOCK;
				-- Tag 9-0
				elseif (Detector == 9 and Reader == 0) then
					CurrentBlockNo = NEW_CASTLE_BLOCK;					
				-- Tag 4-0
				elseif (Detector == 4 and Reader == 0) then
					CurrentBlockNo = STRUTHERS_THRU_TRACK;
				-- Tag 5-0
				elseif (Detector == 5 and Reader == 0) then
					CurrentBlockNo = YOUNGSTOWN_BLOCK;
					end;
				end;
			if (LayoutShowing()) then
				print("Setup : Register the new block's home and distant signals");
				-- 
				local Aspect;			
				local NumLightsHome, NumLightsDistant;
				local HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
				Aspect, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
				local DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
				_, NumLightsDistant = GetAspect(DistantSignal);
				CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
				RegisterCabSignals(HomeSignal, DistantSignal);	
				end;
			end;
		-- done processing the detected engine front tag
		return true;	-- true=continue to process tags, false=stop processing tags
		else
		-- ignore engines that are not me
		return true;	-- true=continue to process tags, false=stop processing tags
		end;
	
	else
	if (CarModel == CABOOSE or CarModel == OBSERVATION) then	-- it's a Caboose or Observation car
		if (BeepOn) then Beep(EXCLAMATION); end;
		printc(clBlue, string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		BumpCounter(C6);
		if (((Detector == 2 or Detector == 4 or Detector == 6 or Detector == 5) and Reader == 1) or ((Detector == 1 or Detector == 4 or Detector == 2 or Detector == 5 or Detector == 6 or Detector == 9 or Detector == 4) and Reader == 0))then
			if (((TagCount) % 5) ~= 0) then
				return Stop("Complete set of 5 tags not detected by a tag reader");
				else
				print("Tag Count OK");
				end;
			end;		
		PlaySound(43, 0, MyEngineNo, MyTIUNo); -- SFS
		-- done processing the detected caboose
		return true;	-- true=continue to process tags, false=stop processing tags
		else	-- it's a car (something other than an engine or a caboose)
		if (BeepOn) then Beep(DEFAULT_BEEP); end;
		printc(clFuchsia, string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		BumpCounter(C5);
		-- done processing the detected car
		return true;	-- true=continue to process tags, false=stop processing tags
		end;
	end;
return true;	-- true=continue to process tags, false=stop processing tags
end	
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
-- this function is called once when the user presses the [STOP] button or the Stop() function is called
SoundOnButton(MyEngineNo, MyTIUNo)
print("All Newly Detected Tags:");
local k, v;
for k,v in  pairs(Tags) do		-- dump out the Tags table
	--                           . . . . . . . . . . . . . . . .
	-- print format:  6619044A  88080101030108011B02202031353536
	print(k .. "  " .. v);
	end
SetSpeed(0, 0, MyEngineNo, MyTIUNo);
Rate(1, 1, 2, MyEngineNo, MyTIUNo);
print("Shutdown in Progress.....");
--
if (StartedFlag) then
	print("Shutting down the Engine");
	Sleep(10);
	EngineShutdown(MyEngineNo, MyTIUNo);
	StartedFlag = false;
	end
RFIDFlag = false;
print(string.format("cleanup() Complete at %.2f", RunTime()));
return true;		-- false=cleanup failed, true=cleanup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function DTOButton(Eng, TIU)
print(string.format("Engine %d Trip Odometer %.3f Smiles", Eng, DTO(Eng, MyTIUNo)));
return true;
end
Function01Name, Function01Label = DTOButton, "Show DTO";
--[[---------------------------------------------------------------------------------------]]
function DODButton(Eng, TIU)
print(string.format("Engine %d Odometer %.3f Smiles", Eng, DOD(Eng, MyTIUNo)));
return true;
end
Function02Name, Function02Label = DODButton, "Show DOD";
--[[---------------------------------------------------------------------------------------]]
function Plus5Button(Eng, TIU)
MySpeed = MySpeed + 5;
SetSpeed(MySpeed, 0, MyEngineNo, MyTIUNo);
SetCounter(C3, MySpeed)
return true;
end
Function03Name, Function03Label = Plus5Button, "Speed +5";
--[[---------------------------------------------------------------------------------------]]
function Minus5Button(Eng, TIU)
MySpeed = MySpeed - 5;
SetSpeed(MySpeed, 0, MyEngineNo, MyTIUNo);
SetCounter(C3, MySpeed)
return true;
end
Function04Name, Function04Label = Minus5Button, "Speed -5";
--[[---------------------------------------------------------------------------------------]]
function SoundOffButton(Eng, TIU)
print("Sound Off");
if (SoundOff == false) then
	-- read up current sound volume
	EngineSoundSave = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
	print(" Sound Off - Engine " .. MyEngineNo .. " Level was " .. EngineSoundSave);
	SoundOff = true;
	SetVolume(MASTER_VOLUME, 0, 3, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function19Name, Function19Label = SoundOffButton, "Sound Off";
--[[---------------------------------------------------------------------------------------]]
function SoundOnButton(Eng, TIU)
print("Sound On");
if (SoundOff == true) then
	print(" Sound On - Engine " .. MyEngineNo .. " Level restored to " .. EngineSoundSave);
	SoundOff = false;
	SetVolume(MASTER_VOLUME, EngineSoundSave, 3, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function20Name, Function20Label = SoundOnButton, "Sound On";
--[[---------------------------------------------------------------------------------------]]
function InitButton(Eng, TIU)
print("Initialize");
print("Set all Switches to Normal postion");
local k, v;
-- set all switches to NORMAL position
for k,v in pairs(Switches) do
	print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
	Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
	Sleep(0.5);
	end
if (Debug() > 3) then
	-- set all switches to REVERSE
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(REVERSE, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	-- set all switches to NORMAL
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	end;

print("Initialization is Complete");
return true;
end
Function06Name, Function06Label = InitButton, "Initialize Layout";
--[[---------------------------------------------------------------------------------------]]
function StartButton(Eng, TIU)
print("Start the Run");
-- start the engine
Rate(1, 1, 0, MyEngineNo, MyTIUNo);
print("setup : Set accell/decell rates to 1");
if (StartedFlag == false) then
	EngineStartUp(MyEngineNo, MyTIUNo);
	print("Starting up the Engine");
	StartedFlag = true;
	Sleep(10);
	end
SetDirection(FORWARD, 0, MyEngineNo, MyTIUNo);

--
RFIDFlag = true;
Whistle(ON, 20, MyEngineNo, MyTIUNo);
Whistle(OFF, 22, MyEngineNo, MyTIUNo);
print("Trip Odometer " .. DTO(MyEngineNo, MyTIUNo)); 	
Sleep(30);

SetSpeed(MySpeed, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, MySpeed)
	
--[[----------------------------------------------------]]
print(string.format("RunTime = %.2f seconds", RunTime()));

return true;
end
Function07Name, Function07Label = StartButton, "Start Run";
--[[---------------------------------------------------------------------------------------]]

function event(FromPC, Type, P1, P2, P3)
print("event() Running - From PC# ", FromPC, " EventType = ", Type, " Parameters = ", P1, " ", P2, " ", P3);
return true;		-- false=event failed, true=event succeeded	
end
--[[---------------------------------------------------------------------------------------]]
--[[
function OccButton(Eng, TIU)
	
return true;
end
Function08Name, Function08Label = OccButton, "8";
--]]
--[[---------------------------------------------------------------------------------------]]
-- Reset
function ResetButton()

print("Reset");
-- stop the engine
SetSpeed(0, 0, MyEngineNo, MyTIUNo);
RFIDFlag = false;
Sleep(5);
--[[-----------------------------]]

--
-- start the engine
if (not StartedFlag) then
	print("Starting up the Engine");
	EngineStartUp(MyEngineNo, MyTIUNo);
	StartedFlag = true;
	Sleep(20);
	end

SetSpeed(MySpeed, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, MySpeed);

return true
end;
Function12Name, Function12Label = ResetButton, "Reset";
--[[---------------------------------------------------------------------------------------]]

function StopButton()
	print("Stop");
	SetSpeed(0, 0, MyEngineNo, MyTIUNo);	
	return true;
end
Function09Name, Function09Label = StopButton, "Stop";
--[[---------------------------------------------------------------------------------------]]

function ResumeSpeedButton()
	print("Resume Speed " .. MySpeed .. " Smph");
	SetSpeed(MySpeed, 0, MyEngineNo, MyTIUNo);	
	return true;
end
Function10Name, Function10Label = ResumeSpeedButton, "Resume Speed";
--[[---------------------------------------------------------------------------------------]]

function ReloadButton()
	print("Reloading layout4.lua");
	package.loaded["layout4"] = nil;	-- unload
	Layout4 = require([[layout4]]);	-- All of the details about the Layout
	return true;
end
Function11Name, Function11Label = ReloadButton, "Reload";
--[[---------------------------------------------------------------------------------------]]

function BeepOffButton()
	print("Beep Off");
	BeepOn = false;
	return true;
end
Function14Name, Function14Label = BeepOffButton, "Beep Off";
--[[---------------------------------------------------------------------------------------]]

function BeepOnButton()
	print("Beep On");
	BeepOn = true;
	return true;
end
Function15Name, Function15Label = BeepOnButton, "Beep On";
--[[---------------------------------------------------------------------------------------]]
--[[
function LLLButton()

	return true;
end
Function05Name, Function05Label = LLLButton, "5";
--]]
--[[---------------------------------------------------------------------------------------]]