--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2021 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]

-- Semicolons are not required in Lua code

function isEngine(EngNo)
	if (EngNo > 0) then return true; end
	return false;
	end
	
function notEngine(EngNo)
	if (EngNo == 0) then return true; end
	return false;
	end	
	
function isTagRear(TagLocation)
	if (TagLocation == TAG_ON_REAR_TRUCK) then return true; end
	return false;
	end

function isTagFront(TagLocation)
	if (TagLocation == TAG_ON_FRONT_TRUCK) then return true; end
	return false;
	end
	
function isDetector(Detector, x)
	if (Detector == x) then return true; end
	return false;
	end	
	
function isReader(Reader, x)
	if (Reader == x) then return true; end
	return false;
	end	
	
function isCarModel(CarModel, x)
	if (CarModel == x) then return true; end
	return false;
	end	

--[[---------------------------------------------------------------------------------------
--
Occupancy Flags are supported as part of the Thinking Engines project.
Up to 100 flags are available to indicate block occupancy. A Zero value
means the block is unoccupied. A non zero value means the block is occupied.
A flag == -1 means the block was manually set occupied.
--
]]
--[[---------------------------------------------------------------------------------------]]
--SetOccupancy
-- Set a value into the block occupancy flag
-- Waits 'forever' for the block to show as unoccupied before it sets occupancy
-- returns two values
-- 1: true on success, false on failure
-- 2: on failure, the block numbers that would not lock, otherwise nil
-- Wait is the number of seconds to wait, default to 1800 seconds (30 minutes)
-- When occupied, the Flag should be set to the engine number of the engine in the block.

function SetOccupancy(block, value, Wait)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in SetOccupancy()");
	end;
if (value == 0) then
	return EmergencyStop("Value in SetOccupancy() is zero, use ClearOccupancy() instead");
	end;
local WaitTime = Wait or 1800;	-- let's say that 30 minutes is 'forever'
-- set the occupancy flag if it is zero or we already have ownership - use lock
while (not TrySetFlag(block, value)) do
	-- wait 'forever'
	if (((WaitTime >> 1) % 10) == 0) then
		print("Waiting for " .. BlockName[block] .. " to clear");
		end;
	Sleep(2);
	WaitTime = WaitTime - 2;
	if (WaitTime <= 0) then
		printc(clRed,"SetOccupancy() Timed out waiting for " .. BlockName[block] .. " to clear"); 
		return false, block;
		end	
	end;
print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- TrySetOccupancy
-- Tries to set a value into the block occupancy flag
-- Only tries once. Returns true on success, false if the block is occupied.

function TrySetOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in TrySetOccupancy()");
	end;
if (value == 0) then
	return EmergencyStop("Value in TrySetOccupancy() is zero, use ClearOccupancy() instead");
	end;
-- set the occupancy flag if it is zero or we already have ownership - use lock
if (not TrySetFlag(block, value)) then
	-- did not get a lock
	print("TrySetOccupancy() flag on " .. BlockName[block] .. " - could not get lock");
	return false;
	end;
print("TrySetOccupancy() flag on " .. BlockName[block] .. " to " .. value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- ClearOccupancy
-- Clear a block occupancy flag to zero if we have ownership, that is,
-- we are occupying the block.

function ClearOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in ClearOccupancy()");
	end;
if (value ~= nil and GetFlag(block) == value) then
	print("ClearOccupancy() flag on " .. BlockName[block]);
	SetFlag(block, 0);
	end;
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- OverrideOccupancy
-- set a block occupancy flag without waiting for lock or checking ownership
-- Can be very dangerous if your script makes a mistake.

function OverrideOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in OverrideOccupancy()");
	end;
if (value == nil) then
	value = 0;
	end;
print("OverrideOccupancy() flag on " .. BlockName[block] .. " to " .. value);
SetFlag(block, value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- GetOccupancy
-- fetch a block occupany flag

function GetOccupancy(block)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in GetOccupancy()");
	end;
return GetFlag(block);
end;
--[[---------------------------------------------------------------------------------------]]
-- MultiSetOccupancy
-- Set a value into several block occupancy flags. All blocks must be unoccupied.
-- Waits 1800 seconds or the given 'Wait' time in seconds for the blocks to show as unoccupied before it sets occupancy
-- Returns two values
-- 1: true on success, false on failure
-- 2: on failure, one of the block numbers that would not lock, otherwise 0
-- Wait is the number of seconds to wait, default to 1800 (30 minutes)
-- blocks is a table of the form {2, 44, 21}

function MultiSetOccupancy(blocks, value, Wait)
if (type(blocks) ~= "table") then return EmergencyStop("MultiSetOccupancy() block not a table"), 0; end
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	if (v < FIRST_BLOCK or v > LAST_BLOCK) then
		return EmergencyStop("Invalid Block Number in MultiSetOccupancy()"), 0;
		end;
	end;
if (value == 0) then
	return EmergencyStop("Value in MultiSetOccupancy() is zero, use ClearOccupancy() instead"), 0;
	end;
local WaitTime = Wait or 1800;	-- let's say 30 minutes is 'forever'
-- set the occupancy flag if it is zero or we already have ownership - use lock
local occupiedblock;
while (true) do
	local result;
	result, occupiedblock = TryMultiSetFlag(blocks, value)
	if (result) then
		break;
		end;
	-- wait almost forever
	if (((WaitTime >> 1) % 10) == 0) then
		print("MultiSetOccupancy() Waiting for " .. BlockName[occupiedblock] .. " to clear");
		end;
	Sleep(2);
	WaitTime = WaitTime - 2;
	if (WaitTime <= 0) then
		printc(clRed,"MultiSetOccupancy() Timed out waiting for " .. BlockName[occupiedblock] .. " to clear"); 
		return false, occupiedblock;
		end
	end;
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	print("MultiSetOccupancy() flag on " .. BlockName[v] .. " to " .. value);
	end;
return true, 0;
end;
--[[---------------------------------------------------------------------------------------]]
-- TryMultiSetOccupancy
-- Set a value into several block occupancy flags. All blocks must be unoccupied.
-- Only tries once
-- returns two values
-- 1: true on success, false on failure
-- 2: on failure, the block number that would not lock, otherwise nil

function TryMultiSetOccupancy(blocks, value)
-- blocks is a table of the form {2, 44, 21}
if (type(blocks) ~= "table") then return EmergencyStop("TryMultiSetOccupancy() block not a table"); end
for k, v in ipairs(blocks) do
	if (v < FIRST_BLOCK or v > LAST_BLOCK) then
		return EmergencyStop("Invalid Block Number in TryMultiSetOccupancy()"), nil;
		end;
	end;
if (value == 0) then
	return EmergencyStop("Value in TryMultiSetOccupancy() is zero, use ClearOccupancy() instead"), nil;
	end;
-- set the occupancy flag if it is zero or we already have ownership - use lock
local success, occupiedblock = TryMultiSetFlag(blocks, value);
if (not success) then
	-- did not get a lock
	print("TryMultiSetOccupancy() flag on " .. BlockName[occupiedblock] .. " - could not get lock");
	return false, occupiedblock;
	end;
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	print("TryMultiSetOccupancy() flag on " .. BlockName[v] .. " to " .. value);
	end;
return true, nil;
end;
--[[---------------------------------------------------------------------------------------]]
-- SingleSetOccupancy
-- Set a value into one of several block occupancy flags
-- Waits 1800 seconds or the given 'Wait' time in seconds for one of the blocks to show as unoccupied before it sets occupancy
-- returns two values
-- 1: true on success, false on failure
-- 2: on success, the block number that locked, otherwise nil
-- Wait is the number of seconds to wait, default to 1800 (30 minutes)
-- blocks is a table of the form {2, 44, 21} or {2}

function SingleSetOccupancy(blocks, value, Wait)
if (type(blocks) ~= "table") then return EmergencyStop("SingleSetOccupancy() block not a table"), 0; end
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	if (v < FIRST_BLOCK or v > LAST_BLOCK) then
		return EmergencyStop("Invalid Block Number in SingleSetOccupancy()"), nil;
		end;
	end;
if (value == 0) then
	return EmergencyStop("Value in SingleSetOccupancy() is zero, use ClearOccupancy() instead"), nil;
	end;
local WaitTime = Wait or 1800;	-- let's say that 30 minutes is 'forever'
-- set the occupancy flag if it is zero or we already have ownership - use lock
local occupiedblock;
while (true) do
	for k,v in ipairs(blocks) do
		-- lock the first block that becomes available
		local result = TrySetFlag(v, value)
		if (result) then
			occupiedblock = v;
			print("SingleSetOccupancy() flag on " .. BlockName[occupiedblock] .. " to " .. value);
			return true, occupiedblock;
			end;
		if (((WaitTime >> 1) % 10) == 0) then
			print("SingleSetOccupancy() Waiting for " .. BlockName[v] .. " to clear");
			end;
		end;
	-- wait almost forever
	Sleep(2);
	WaitTime = WaitTime - 2;
	if (WaitTime <= 0) then
		break;
		end		
	end;

	for k,v in ipairs(blocks) do
		printc(clRed,"SingleSetOccupancy() Timed out waiting for " .. BlockName[v] .. " to clear"); 
		end;

return false, nil;
end;
--[[---------------------------------------------------------------------------------------]]

