--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2022 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
require([[defines]]);
require([[functions]]);
--
--[[
When using this module, all variables (including functions!) must be declared through 
a regular assignment (even assigning nil will do) in a strict scope before being used 
anywhere or assigned to inside a nested scope.
https://github.com/lua-stdlib/strict
]]
local _ENV = require 'std.strict' (_G)	-- strict.lua will detect undeclared global variables
local _;
--
--[[---------------------------------------------------------------------------------------]]
local title = "Signals EB"; -- Eastbound
local MyDirection = EB;
local OppDirection = WB;
local Headway = false;	-- Set to true to enable Headway decision making
--[[
Headway decision making allows the engine to move into the next block
if that block is unoccupied and the signal for that block shows a clear aspect.

This is different than the non-headway condition. Then the engine can only move
into the next block if that block and all blocks into the next siding
show a clear aspect.

--]]
local Signals = false;	-- Set to true to incorporate signals into the decision making
--[[---------------------------------------------------------------------------------------]]
local TagCount = 0;
-- Counter Labels
Counter01Label = "tags()";
Counter03Label = "Block Speed Limit";
local C3 = COUNTER03;
Counter04Label = "Target Distance";
local C4 = COUNTER04;
Counter05Label = "Stopping Distance";
local C5 = COUNTER05;
Counter06Label = "Stop Delay";
local C6 = COUNTER06;

local Tags = {};		-- table to hold all of the tags detected in sequence
local SoundOn = true;
local LastSentSpeed = 0;

local StartBlockNo, EndBlockNo;
local CurrentBlockNo, LastBlockNo, NextBlockNo;
local StartedFlag = false;
local RFIDFlag = false;
local ResetinProgress = false;

local LLLoop = true;	-- start in looping mode
local Layout4;
local MyTIUNo;
local MyEngineNo = 16;
Title(title, MyEngineNo);
local SoundOff = false;
local EngineSoundSave;
local BeepOn = false;
local BlockSpeedLimit = 0;	-- Smph
local LastBlockSpeedLimit = 0;	-- Smph
local ExpectedReader = 0;	-- check that we get the next expected tag reader
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func90 = nil;		-- Split New Castle Block
local Func40 = nil;
local Func50 = nil;
local Func60 = nil;
local Func61 = nil;
local Func100 = nil;	-- Split Alquippa block
local Func101 = nil;	-- Split Beaver Falls Block

-- These labels appear on each of the tabs for the Function Buttons
Tab01Label = "Control";
Tab02Label = "Support";
Tab03Label = "Debug";
--[[---------------------------------------------------------------------------------------]]

function StopAfterDelay(StoppingBlockNo, EngineNo, direction, LastBlockSpeedLimit, BSL)
-- Calculate the stop delay based on parameters
--
-- LastBlockSpeedLimit is the speed at which the block was entered
-- BlockSpeedLimit is the speed to which the engine is being slowed before it is asked to stop
-- LastBlockSpeedLimit and BlockSpeedLimit may be equal
-- Either one may be greater than the other
--
-- Returns the stop delay in seconds and the target distance in inches
local Target_Station;
local Target_Distance;
local StopDelay;

local BlockSpeedLimit = BSL or LastBlockSpeedLimit;

Target_Station = BlockName[StoppingBlockNo];
if (EngineNo == 7 or EngineNo == 13) then
	Target_Distance = Layout4.STOPBlock(StoppingBlockNo, direction, PASSENGER);	--Passenger StoppingDistance;
	else
	Target_Distance = Layout4.STOPBlock(StoppingBlockNo, direction, FREIGHT);	--Freight StoppingDistance;
	end
if (Target_Distance == nil) then
	Target_Distance = 0;
	StopDelay = 0;
	return EmergencyStop("SAD() : Target_Distance missing in Layout description");
	end
SetCounter(C4, Target_Distance);
--print(BlockName[StoppingBlockNo] .. " " .. direction .. ", Target Distance = " .. Target_Distance);
--
-- First calculate the distance from entrance of the block until the engine gets down to Intermediate Speed
-- Note: this distance may be zero if the two speeds are the same
local StoppingDistance = Layout4.STOPDistance(EngineNo, BlockSpeedLimit);
-- then calculate the extra distance caused by the engine slowing down or the sufet distance caused
-- by the engine speeding up from the speed of the last block with the speed of this block.
local StepDistance = Layout4.STOPDistance(EngineNo, LastBlockSpeedLimit) - StoppingDistance;
print(string.format("SAD() : LastBlockSpeedLimit = %d Smph - Stopping Distance = %.3f in - BlockSpeedLimit = %d Smph - Stopping Distance = %.3f in - Step Distance = %.3f in",
	LastBlockSpeedLimit, Layout4.STOPDistance(EngineNo, LastBlockSpeedLimit), BlockSpeedLimit, StoppingDistance, StepDistance));
-- Subtract these two values from the target distance and calcuate the Stop Delay
--
-- Equation for Stop Delay calculation
-- Time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr / (12 in/ft * 5280 ft/mile * X Smiles/hr)
StopDelay =  (Target_Distance - StoppingDistance - StepDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
-- if the StopDelay value comes out negative, that means we have to stop
-- very quickly. If that happens, set the Decell rate very high and then
-- set the speed to zero.
SetCounter(C5, StoppingDistance);
SetCounter(C6, StopDelay);
print(string.format("SAD() : %s : Target Distance = %d in - Stopping Distance = %.3f in - Speed %d Smph = Stop Delay %.3f seconds",
	BlockName[StoppingBlockNo], Target_Distance, StoppingDistance, BlockSpeedLimit, StopDelay));
if (StopDelay < 0) then
	StopDelay = 0;
	return EmergencyStop("SAD() : Engine cannot stop in time (Stopping Distance > Target_Distance)");
	end;
-- Stop delay any positive value including 0
if (StopDelay == 0) then
	Rate(8, 8, StopDelay, EngineNo, MyTIUNo);	-- stop as quickly as possible
	end;
LastSentSpeed = SetSpeed(0, StopDelay, EngineNo, MyTIUNo);	-- stop engine;
if (StopDelay == 0) then
	Rate(Layout4.AccDecRate, Layout4.AccDecRate, StopDelay, EngineNo, MyTIUNo);
	end;
print(string.format("Set speed to 0 after %.3f seconds", StopDelay))
return StopDelay, Target_Distance;
end;
--[[---------------------------------------------------------------------------------------]]
function createMainLine2Passing(flag)
	local WaitforCaboose = flag;
	--local SwitchClearingDistance = SwDistance or 24;	
	local EngineStopped = false;
	local StopDelay = 0;
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance = 0;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;
	local HomeSignal, DistantSignal;	
	local TrainTime = 0;
	local function MainLine2Passing(DetectorID, EngineNo, TagLocation, CarModel, Normal_Track, Reverse_Track, ThisSwitch)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			
--[[--- Engine Detected -----]]
			
			TrainTime = RunTime();	-- Engine detected time	
			print("ML2P : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			local Spd = GetSpeed(MyEngineNo, MyTIUNo);	-- get speed as engine covers tag
			if (Spd ~= BlockSpeedLimit) then
				return EmergencyStop("ML2P : Speed over tag = " .. Spd .. " Smph - expected = " .. BlockSpeedLimit .. " Smph");
				end;			
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			-- initialize block local variables
			EngineStopped = false;
			GotBlock = false;
			--	
			-- Register the new block's home and distant signals
			local Aspect;			
			local NumLightsHome, NumLightsDistant;
			HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
			Aspect, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
			DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
			_, NumLightsDistant = GetAspect(DistantSignal);
			CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
			RegisterCabSignals(HomeSignal, DistantSignal);
			-- Adjust speed for the current block
			LastBlockSpeedLimit = BlockSpeedLimit;
			BlockSpeedLimit = AdjustSpeed(CurrentBlockNo, Aspect);	-- adjust speed for some randomness in high speed blocks
			SetCounter(C3, BlockSpeedLimit);
			LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			print(string.format("ML2P : %s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));			
			print ("ML2P : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 1 ---------------------------------------------------------------------------------------------------
			-- check if home signal is AsStopandProceed. If it is, stop in this block. Set EngineStopped = true.
			-- NOTE: Mainline2Passing home signal can only be a permissive signal			
			if (Signals and (Aspect == AsStopandProceed)) then
				-- stop in this block
				print("ML2P : Home Signal aspect is Stop and Proceed. Stop the engine");
				-- stop in CurrentBlockNo
				StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
				EngineStopped = true;
				GotBlock = false;
				StopTime = RunTime();				
				end;
			if (Signals and (Aspect == AsStopSignal)) then
				return EmergencyStop("ML2P : Home Signal aspect is an unexpected absolute signal.");
				end;
-----------------------------------------------------------------------------------------------------]]
			-- set next block number
			if (MyDirection == SuperiorDirection) then
				NextBlockNo = Normal_Track;		-- Westbound trains are superior, take the through track
				else
				NextBlockNo = Reverse_Track;	-- Eastbound trains are inferior, take the passing/station track
				end;
			-- If we didn't stop on a stop signal or stop and proceed, then attempt to obtain occupancy lock
			if (EngineStopped == false) then
				-- check for occupancy of thru track or passing/station track
				-- choose based on superiority.
				-- If we can't get a lock, schedule a stop once the entire train is in the block.
				if (MyDirection == SuperiorDirection) then
					if (TrySetOccupancy(Normal_Track, MyEngineNo)) then	-- Thru track
						-- Got a lock on the next block
						StopDelay = 0;
						EngineStopped = false;
						GotBlock = true;
						StopTime = 0;
						else
						-- if we can't get a lock, stop in CurrentBlockNo
						StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit,BlockSpeedLimit)
						EngineStopped = true;
						GotBlock = false;
						StopTime = RunTime();
						end
					else	-- MyDirection ~= SuperiorDirection
					if (TrySetOccupancy(Reverse_Track, MyEngineNo)) then	-- passing/station track
						-- got a lock on the next block
						StopDelay = 0;
						EngineStopped = false;
						GotBlock = true;
						StopTime = 0;
						else
						-- if we can't get a lock, stop in CurrentBlockNo
						StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit,BlockSpeedLimit)
						EngineStopped = true;
						GotBlock = false;
						StopTime = RunTime();
						end;
					end;
				end;
--[=[
	----- NOTE : all blocks are longer than 50 inches
			-- engine was stopped quickly
			if (not GotBlock and Target_Distance <= 50) then
				-- Caboose may not have passed over detector yet and it may occupy two blocks
				-- wait for one block to be locked (default wait 10 minutes)
				-- wait for either Normal_Track or Reverse_Track to lock
				local result;
				if (MyDirection == SuperiorDirection) then
					result, NextBlockNo = SingleSetOccupancy({Normal_Track}, MyEngineNo);
					if (not result) then
						return EmergencyStop("ML2P : Timed out waiting for block: " .. BlockName[Normal_Track]);
						end;
					else
					result, NextBlockNo = SingleSetOccupancy({Reverse_Track}, MyEngineNo);
					if (not result) then
						return EmergencyStop("ML2P : Timed out waiting for block: " .. BlockName[Reverse_Track]);
						end;
					end;
				
				GotBlock = true;	-- indicate Block is locked
				if (EngineStopped) then
					-- start moving if stopped 
					local Aspect = GetCabAspect(HOMESIGNAL);
					print ("ML2P : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 2 -----------------------------------------------------------------------------------------------
				if (Signals and (Aspect == AsStopandProceed)) then
					-- stop in this block
					print("ML2P : Home Signal aspect is Stop and Proceed. Engine is stopped, waiting for home signal to clear");
					local WaitTime = 1800;	-- seconds
					while (Aspect == AsStopandProceed) do
						-- wait 'forever'
						if (((WaitTime >> 1) % 10) == 0) then
							print("Waiting for home signal at block " .. BlockName[CurrentBlockNo] .. " to clear");
							end;
						Sleep(2);
						WaitTime = WaitTime - 2;
						if (WaitTime <= 0) then
							return EmergencyStop("Timed out waiting for home signal at block " .. BlockName[CurrentBlockNo] .. " to clear"); 
							end;	
						end;				
					end;
					-- NOTE: Mainline2Passing can only be permissive signal
					if (Signals and Aspect == AsStopSignal) then
						return EmergencyStop("ML2P : Home signal is an unexpected absolute signal");	-- Error in layout design
						end;
-----------------------------------------------------------------------------------------------------]]
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 5;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
						end;
					--print("ML2P : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
					LastBlockSpeedLimit = BlockSpeedLimit;
					LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					print(string.format("ML2P : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here
					end;
				end;
--]=]
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			DisplayBlockNames("ML2P : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;
			
--[[--- Caboose Detected -----]]

		elseif (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION) and WaitforCaboose) then
			-- in the elseif block, use the local copy of CurrentBlockNo and LastBlockNo
			-- ASAP Once the caboose is detected (without any clearing delay) return switch at entrance of passing/station track back
			-- to NORMAL if it is not already at NORMAL.
			print("ML2P : " .. DetectorID .. "End of Train after " .. RunTime() - TrainTime .. " seconds (Stop Delay = " .. StopDelay .. ")");
			--
			if (ThisSwitch ~= nil) then
				Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
				print("ML2P : " .. SwitchName[ThisSwitch] .. " to NORMAL");
				end;
			--
			--print("MyCurrentBlockNo " .. MyCurrentBlockNo .. " MyDirection " .. MyDirection);
			--print("Opposite Direction " .. OppDirection);
			--print("Detector = " .. DETECTOR .. " Reader = " .. READER);		
			--print(" Detector = " .. Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR));
			--print(" Reader = " ..   Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER));
			ExpectedReader = Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR) * 10 + Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER);
			print("Expected Reader Changed to " .. ExpectedReader);
			--
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (not GotBlock) then
				-- wait for block to be locked (default wait 10 minutes)
				-- wait for MyNextBlockNo to be locked (either the thru or passing/station track)
				local result;
				-- set occupancy flag in MyNextBlockNo
				if (MyNextBlockNo == Normal_Track) then
					result, MyNextBlockNo = SingleSetOccupancy({Normal_Track}, MyEngineNo);
					if (not result) then
						return EmergencyStop("ML2P : Timed out waiting for block: " .. BlockName[Normal_Track]);
						end;
					end;
				if (MyNextBlockNo == Reverse_Track) then
					result, MyNextBlockNo = SingleSetOccupancy({Reverse_Track}, MyEngineNo);
					if (not result) then
						return EmergencyStop("ML2P : Timed out waiting for block: " .. BlockName[Reverse_Track]);
						end;
					end;
				GotBlock = true;	-- indicate needed Block is locked
				end;
				

			if (EngineStopped) then
				-- start moving if stopped 
				-- Check if Stop and Proceed aspect is showing
				local Aspect = GetCabAspect(HOMESIGNAL);
				print ("ML2P : Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);				
---[[--TODO 3 -----------------------------------------------------------------------------------------------
				if (Signals and (Aspect == AsStopandProceed)) then
					-- if we are stopped in this block due to Stop and Proceed aspect, wait 10 seconds before restarting
					Sleep(10);
					end;
				-- NOTE: Mainline2Passing can only be permissive signal
-----------------------------------------------------------------------------------------------------]]

				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 5;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
					end;			
				--print("ML2P : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
				LastBlockSpeedLimit = BlockSpeedLimit;
				LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				print(string.format("ML2P : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));		
				EngineStopped = false;	-- indicate that we handled the stop here
				end;
				
			WaitforCaboose = false;
			end;
	return true;
	end;

return MainLine2Passing;
end;
--[[---------------------------------------------------------------------------------------]]
function createMainLine2MainLine(flag)
	local WaitforCaboose = flag;
	local EngineStopped = false;
	local StopDelay = 0
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance = 0;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;
	local TrainTime = 0;
	local function MainLine2MainLine(DetectorID, EngineNo, TagLocation, CarModel, ThisSwitch, Block)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (type(Block) ~= "table") then
			return EmergencyStop("ML2ML Block not a table");
			end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			
--[[--- Engine Detected -----]]
			
			TrainTime = RunTime();	-- Engine detected time	
			print("ML2ML " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			local Spd = GetSpeed(MyEngineNo, MyTIUNo);	-- get speed as engine covers tag
			if (Spd ~= BlockSpeedLimit) then
				return EmergencyStop("ML2ML : Speed over tag = " .. Spd .. " Smph - expected = " .. BlockSpeedLimit .. " Smph");
				end;
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			-- initialize block local variables
			EngineStopped = false;
			GotBlock = false;
			--
			-- Register the new block's home and distant signals
			local Aspect;			
			local NumLightsHome, NumLightsDistant;
			local HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
			Aspect, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
			local DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
			_, NumLightsDistant = GetAspect(DistantSignal);
			CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
			RegisterCabSignals(HomeSignal, DistantSignal);
			-- Adjust speed for the current block
			LastBlockSpeedLimit = BlockSpeedLimit;
			BlockSpeedLimit = AdjustSpeed(CurrentBlockNo, Aspect);	-- adjust speed for some randomness in high speed blocks
			SetCounter(C3, BlockSpeedLimit);
			LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			print(string.format("ML2ML : %s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));
			print ("ML2ML : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 1 -----------------------------------------------------------------------------------------------
			-- Check if home signal is AsStopandProceed. If it is, stop in this block. Set EngineSTopped = true.
			-- NOTE: MainLine2MainLine can only be permissive signal
			if (Signals and (Aspect == AsStopandProceed)) then
				-- stop in this block
				print("ML2ML : Home Signal aspect is Stop and Proceed. Stop the engine");
				-- stop in CurrentBlockNo
				StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
				EngineStopped = true;
				GotBlock = false;
				StopTime = RunTime();				
				end;
			if (Signals and (Aspect == AsStopSignal)) then
				return EmergencyStop("ML2ML : Home Signal is an unexpected absolute signal.");
				end;
-----------------------------------------------------------------------------------------------------]]
			NextBlockNo = Block[1];	-- set NextBlockNo to Block[1]
			-- If we didn't stop on a stop signal or stop and proceed, then attempt to obtain occupancy lock
			if (EngineStopped == false) then		
				-- no engine entrance switch settings
				-- moving into a block requires the block to be empty (== 0) or
				-- be already reserved for this engine (== EnginNo)
				if (not TryMultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then
					-- if we can't get a lock, stop in CurrentBlockNo
					StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
					EngineStopped = true;	-- engine was stopped
					GotBlock = false;
					StopTime = RunTime();
					else
					-- got a lock on the needed block(s), continue on
					StopDelay = 0;
					EngineStopped = false;	-- engine was not stopped
					GotBlock = true;
					StopTime = 0;
					end;
				end;
--[=[
	----- NOTE : all blocks are longer than 50 inches
			-- engine was stopped quickly
			if (not GotBlock and Target_Distance <= 50) then
				-- Caboose may not have passed over detector yet and train may occupy two blocks
				-- wait for one/all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks						
					return EmergencyStop("ML2ML : Timed out waiting for " .. BlockName[NextBlockNo]);
					end
				GotBlock = true;	-- indicate Block is locked
				if (EngineStopped) then
					-- start moving if stopped 	
					local Aspect = GetCabAspect(HOMESIGNAL);
					print ("ML2ML : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 2 -----------------------------------------------------------------------------------------------
					if (Signals and (Aspect == AsStopandProceed)) then
						-- stop in this block
						print("ML2ML : Home Signal aspect is Stop and Proceed. Engine is stopped, waiting for home signal to clear");
						local WaitTime = 1800;	-- seconds
						while (Aspect == AsStopandProceed) do
							-- wait 'forever'
							if (((WaitTime >> 1) % 10) == 0) then
								print("Waiting for home signal at block " .. BlockName[CurrentBlockNo] .. " to clear");
								end;
							Sleep(2);
							WaitTime = WaitTime - 2;
							if (WaitTime <= 0) then
								return EmergencyStop("Timed out waiting for home signal at block " .. BlockName[CurrentBlockNo] .. " to clear"); 
								end;	
							end;
						end;
					-- NOTE: MainLine2MainLine can only be permissive signal
					if (Signals and Aspect == AsStopSignal) then
						return EmergencyStop("ML2ML : home signal is an unexpected absolute signal");
						end;
-----------------------------------------------------------------------------------------------------]]
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 5;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
						end;				
					--print("ML2ML : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
					LastBlockSpeedLimit = BlockSpeedLimit;
					LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					print(string.format("ML2ML : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here
					end;
				end;
--]=]				
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			DisplayBlockNames("ML2ML : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;
			return true;
			
--[[--- Caboose Detected -----]]

		elseif (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION) and WaitforCaboose) then
			print("ML2ML : " .. DetectorID .. "End of Train after " .. RunTime() - TrainTime .. " seconds (Stop Delay = " .. StopDelay .. ")");
			-- on any MainLine exit readers, the switch is before the tag reader so 
			-- the SwitchClearingDistance is zero
			-- ASAP Once the caboose is detected (without any clearing delay) return switch at exit of passing/station track back
			-- to NORMAL - this switch was set as result of the engine entering that last block
			--
			if (ThisSwitch ~= nil) then
				Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
				print("ML2ML : " .. SwitchName[ThisSwitch] .. " to NORMAL");
				end;
			--
			--print("MyCurrentBlockNo " .. MyCurrentBlockNo .. " MyDirection " .. MyDirection);
			--print("Opposite Direction " .. OppDirection);
			--print("Detector = " .. DETECTOR .. " Reader = " .. READER);		
			--print(" Detector = " .. Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR));
			--print(" Reader = " ..   Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER));
			ExpectedReader = Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR) * 10 + Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER);
			print("Expected Reader Changed to " .. ExpectedReader);			
			--				
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy		
				end;
			if (not GotBlock) then
				-- wait for one/all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks			
					return EmergencyStop("ML2ML : Timed out waiting for " .. BlockName[MyNextBlockNo]);
					end;
				GotBlock = true;
				end;
			if (EngineStopped) then
				-- start moving if stopped 
				local Aspect = GetCabAspect(HOMESIGNAL);
				print ("ML2ML : Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 3 -----------------------------------------------------------------------------------------------
				if (Signals and (Aspect == AsStopandProceed)) then
					-- if we are stopped in this block due to Stop and Proceed aspect, wait 10 seconds before restarting
					Sleep(10);
					end;
				-- NOTE: MainLine2MainLine can only be permissive signal
-----------------------------------------------------------------------------------------------------]]
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 5;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
					end;				
				--print("ML2ML : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
				LastBlockSpeedLimit = BlockSpeedLimit;
				LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				print(string.format("ML2ML : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));
				EngineStopped = false;	-- indicate that we handled the stop here
				end;
				
			WaitforCaboose = false;
			end;
	return true;
	end;

return MainLine2MainLine;
end;
--[[---------------------------------------------------------------------------------------]]
function createPassing2MainLine(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance or 0;
	local EngineStopped = false;
	local StopDelay = 0;
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance = 0;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;	
	local TrainTime = 0;	
	local function Passing2MainLine(DetectorID, EngineNo, TagLocation, CarModel, Normal_Track, Reverse_Track, ThisSwitch, NextSwitch, Block)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (type(Block) ~= "table") then
			return EmergencyStop("P2ML : Block not a table");
			end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			
--[[--- Engine Detected -----]]
			
			TrainTime = RunTime();	-- Engine detected time	
			print("P2ML : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			local Spd = GetSpeed(MyEngineNo, MyTIUNo);	-- get speed as engine covers tag
			if (Spd ~= BlockSpeedLimit) then
				return EmergencyStop("P2ML : Speed over tag = " .. Spd .. " Smph - expected = " .. BlockSpeedLimit .. " Smph");
				end;
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			-- initialize block local variables
			EngineStopped = false;
			GotBlock = false;
			--				
			-- Register the new block's home and distant signals
			local Aspect;
			local NumLightsHome, NumLightsDistant;
			local HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
			Aspect, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
			local DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
			_, NumLightsDistant = GetAspect(DistantSignal);
			CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
			RegisterCabSignals(HomeSignal, DistantSignal);
			-- Adjust speed for the current block
			LastBlockSpeedLimit = BlockSpeedLimit;
			BlockSpeedLimit = AdjustSpeed(CurrentBlockNo, Aspect);	-- adjust speed for some randomness in high speed blocks
			SetCounter(C3, BlockSpeedLimit);
			LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			print(string.format("P2ML : %s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));
			print ("P2ML : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 1 -----------------------------------------------------------------------------------------------
			-- Check if home signal is AsStopSignal. If it is, stop in this block. Set EngineSTopped = true.
			-- NOTE: Passing2MainLine is the only case where an absolute signal can and must appear
			if (Signals and (Aspect == AsStopSignal)) then
				print("P2ML : Home Signal aspect is Stop Signal. Stop the engine");
				-- stop in CurrentBlockNo
				StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
				EngineStopped = true;
				GotBlock = false;
				StopTime = RunTime();				
				end;
			if (Signals and (Aspect == AsStopandProceed)) then
				return EmergencyStop("P2ML : Home Signal is an unexpected permissive signal.");
				end;				
-----------------------------------------------------------------------------------------------------]]
			--
			-- ASAP, set the switch at entrance to passing/station track
			--
			if (ThisSwitch ~= nil) then
				if (CurrentBlockNo == Normal_Track) then
					Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("P2ML : " .. SwitchName[ThisSwitch] .. " to NORMAL");
					else
					Switch(REVERSE, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("P2ML : " .. SwitchName[ThisSwitch] .. " to REVERSE");
					end;
				end;
			NextBlockNo = Block[1];	-- set NextBlockNo to Block[1]
			-- If we didn't stop on a stop signal or stop and proceed, then attempt to obtain occupancy lock
			if (EngineStopped == false) then
				-- moving into a block requires the block to be empty (== 0) or
				-- be already reserved for this engine (== MyEngineNo)
				if (not TryMultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then
					-- if we can't get a lock, stop in CurrentBlockNo
					StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
					EngineStopped = true;	-- engine was stopped
					GotBlock = false;
					StopTime = RunTime();
					else
					-- got a lock on the needed block(s), continue on
					StopDelay = 0;
					EngineStopped = false;	-- engine was not stopped
					GotBlock = true;
					StopTime = 0;
					end;
				end;
--[=[
	----- NOTE : all blocks are longer than 50 inches
			-- Engine was stopped quickly
			if (not GotBlock and Target_Distance <= 50) then
				-- Caboose may not have passed over detector yet and train may occupy two blocks
				-- wait for one/all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks			
					return EmergencyStop("P2ML : Timed out waiting for " .. BlockName[NextBlockNo]);
					end
				GotBlock = true;	-- indicate Block is locked	
				if (EngineStopped) then
					-- start moving if stopped 
					local Aspect = GetCabAspect(HOMESIGNAL);
					print ("P2ML : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 2 -----------------------------------------------------------------------------------------------
					-- Check that home signal is not AsStopSignal
					-- NOTE: Passing2MainLine is the only case where an absolute signal can and must appear
					if (Signals and Aspect == AsStopSignal) then
						print("P2ML : Stop Signal");
						WaitforStopSignaltoClear();		
						end;				
-----------------------------------------------------------------------------------------------------]]
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 5;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
						end;
					--print("P2ML : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
					LastBlockSpeedLimit = BlockSpeedLimit;
					LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					print(string.format("P2ML : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here	
					end;
				end;
--]=]					
			if (GotBlock) then
				-- set NextSwitch switch
				if (CurrentBlockNo == Normal_Track) then
					Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
					print("P2ML : Next Switch " .. SwitchName[NextSwitch] .. " to NORMAL");
				elseif (CurrentBlockNo == Reverse_Track) then
					Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
					print("P2ML : NextSwitch " .. SwitchName[NextSwitch] .. " to REVERSE");
					end;
				end;
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			DisplayBlockNames("P2ML : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;
			
--[[--- Caboose Detected -----]]

		elseif (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION) and WaitforCaboose) then
			-- in the elseif block, use the local copy of CurrentBlockNo and LastBlockNo
			print("P2ML : " .. DetectorID .. "End of Train after " .. RunTime() - TrainTime .. " seconds (Stop Delay = " .. StopDelay .. ")");
			if (SwitchClearingDistance > 0) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
				print(string.format("P2ML : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], SwitchClearingDistance, BlockSpeedLimit, SleepDelay));
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				print("P2ML : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				end;
			--
			-- Once we reach the switch clearing distance, return switch at entrance to passing/station track back
			-- to NORMAL - this switch was set as result of the engine entering the block
			--
			if (ThisSwitch ~= nil) then
				Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
				print("P2ML : " .. SwitchName[ThisSwitch] .. " to NORMAL");
				end;
			--
			--print("MyCurrentBlockNo " .. MyCurrentBlockNo .. " MyDirection " .. MyDirection);
			--print("Opposite Direction " .. OppDirection);
			--print("Detector = " .. DETECTOR .. " Reader = " .. READER);		
			--print(" Detector = " .. Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR));
			--print(" Reader = " ..   Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER));
			ExpectedReader = Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR) * 10 + Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER);
			print("Expected Reader Changed to " .. ExpectedReader);
			--				
			if (MyLastBlockNo ~= 0) then			
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy		
				end;
			if (not GotBlock) then
				-- wait for one/all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks			
					return EmergencyStop("P2ML : Timed out waiting for " .. BlockName[MyNextBlockNo]);
					end;
				GotBlock = true;	-- indicate Block is locked
				-- set next switch (passing/station track exit) based on current block
				if (NextSwitch ~= nil and GotBlock) then
					-- set NextSwitch switch
					if (MyCurrentBlockNo == Normal_Track) then
						Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("P2ML : Next Switch " .. SwitchName[NextSwitch] .. " to NORMAL");
						end;
					if (MyCurrentBlockNo == Reverse_Track) then
						Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("P2ML : Next Switch " .. SwitchName[NextSwitch] .. " to REVERSE");
						end;			
					end;
				end;

			if (EngineStopped) then
				-- start moving if stopped 
				local Aspect = GetCabAspect(HOMESIGNAL);
				print ("P2ML : Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 3 -----------------------------------------------------------------------------------------------
				-- check if home signal is AsStopSignal
				-- NOTE: Passing2Mainline is the only case where an absolute signal can and must appear
				if (Signals and Aspect == AsStopSignal) then
					print("P2ML : Stop Signal");
					WaitforStopSignaltoClear();		
					end;
-----------------------------------------------------------------------------------------------------]]
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 5;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
					end;			
				--print("P2ML : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
				LastBlockSpeedLimit = BlockSpeedLimit;
				LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				print(string.format("P2ML : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));
				EngineStopped = false;	-- indicate that we handled the stop here
				end;

			WaitforCaboose = false;				
			end;
	return true;
	end;

return Passing2MainLine;
end;
--[[---------------------------------------------------------------------------------------]]

function createMainLine2Yard(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance or 24;		
	local EngineStopped = false;
	local StopDelay = 0;
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;
	local TrainTime = 0;	
	local function MainLine2Yard(DetectorID, EngineNo, TagLocation, CarModel, Yard1, Yard2, ThisSwitch, Block)
		if (type(Block) ~= "table") then
			return EmergencyStop("ML2Y : Block not a table");
			end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			
--[[--- Engine Detected -----]]
			
			TrainTime = RunTime();	-- Engine detected time	
			print("ML2Y : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			local Spd = GetSpeed(MyEngineNo, MyTIUNo);	-- get speed as engine covers tag
			if (Spd ~= BlockSpeedLimit) then
				return EmergencyStop("ML2Y : Speed over tag = " .. Spd .. " Smph - expected = " .. BlockSpeedLimit .. " Smph");
				end;			
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			-- initialize block local variables
			EngineStopped = false;
			GotBlock = false;
			--	
			-- Register the new block's home and distant signals
			local Aspect;
			local NumLightsHome, NumLightsDistant;
			local HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
			Aspect, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
			local DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
			_, NumLightsDistant = GetAspect(DistantSignal);
			CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
			RegisterCabSignals(HomeSignal, DistantSignal);
			-- Adjust speed for the current block
			LastBlockSpeedLimit = BlockSpeedLimit;
			BlockSpeedLimit = AdjustSpeed(CurrentBlockNo, Aspect);	-- adjust speed for some randomness in high speed blocks
			SetCounter(C3, BlockSpeedLimit);
			LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			print(string.format("ML2Y : %s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));	
			print ("ML2Y : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 1 ---------------------------------------------------------------------------------------------------
			-- Check if home signal is AsStopandProceed. If it is, stop in this block. Set EngineSTopped = true.
			-- Then go on to check TrySetOccupancy and if we get it, start moving again.
			-- NOTE: MainLine2Yard can only be a permissive signal
			if (Signals and (Aspect == AsStopandProceed)) then
				-- stop in this block
				print("ML2Y : Home Signal aspect is Stop and Proceed. Stop the engine");
				-- stop in CurrentBlockNo
				StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
				EngineStopped = true;
				GotBlock = false;
				StopTime = RunTime();				
				end;
			if (Signals and (Aspect == AsStopSignal)) then
				return EmergencyStop("ML2Y : Home Signal is an unexpected absolute signal.");
				end;
-----------------------------------------------------------------------------------------------------]]
			NextBlockNo = Block[1];		-- set NextBlockNo to Block[1]				
			-- If we didn't stop on a stop signal or stop and proceed, then attempt to obtain occupancy lock
			if (EngineStopped == false) then
				-- check for occupancy clear Yard2 or Yard1 - Ending block must be unoccupied otherwise stop engine
				if (TrySetOccupancy(NextBlockNo, MyEngineNo)) then
					StopDelay = 0;
					EngineStopped = false;
					GotBlock = true;
					StopTime = 0;
					else
					-- if we can't get a lock, stop in CurrentBlockNo
					StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
					EngineStopped = true;
					GotBlock = false;
					StopTime = RunTime();
					end;
				end;
--[=[
	----- NOTE : all blocks are longer than 50 inches			
			-- engine was stopped quickly
			if (not GotBlock and Target_Distance <= 50) then
				-- Caboose may not have passed over detector yet and it may occupy two blocks
				-- wait for all blocks to be locked (default wait 10 minutes)
				-- wait for either Normal_Track or Reverse_Track to clear
				SetOccupancy(NextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo
				
				GotBlock = true;	-- indicate Block is locked
				if (EngineStopped) then
					-- start moving if stopped 				
					local Aspect = GetCabAspect(HOMESIGNAL);
					print ("ML2Y : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 2 -----------------------------------------------------------------------------------------------
					-- NOTE: MainLine2Yard can only be a permissive signal
					if (Signals and Aspect == AsStopSignal) then
						return EmergencyStop("ML2Y : home signal is an unexpected absolute signal");
						end;					
-----------------------------------------------------------------------------------------------------]]
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 5;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
						end;
					--print("ML2Y : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
					LastBlockSpeedLimit = BlockSpeedLimit;
					LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					print(string.format("ML2Y : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here
					end;
				end;
--]=]					
			if (GotBlock) then	
				-- set ThisSwitch switch
				if (NextBlockNo == Yard1) then
					Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("ML2Y : Switch " .. SwitchName[ThisSwitch] .. " to NORMAL");
					end;
				if (NextBlockNo == Yard2) then
					Switch(REVERSE, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("ML2Y : Switch " .. SwitchName[ThisSwitch] .. " to REVERSE");
					end;
				end;
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			DisplayBlockNames("ML2Y : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;
			
--[[--- Caboose Detected -----]]

		elseif (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION) and WaitforCaboose) then
			print("ML2Y : " .. DetectorID .. "End of Train after " .. RunTime() - TrainTime .. " seconds (Stop Delay = " .. StopDelay .. ")");
			if (SwitchClearingDistance > 0) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
				print(string.format("ML2Y : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], SwitchClearingDistance, BlockSpeedLimit, SleepDelay));
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				print("ML2Y : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				end;
			--
			--print("MyCurrentBlockNo " .. MyCurrentBlockNo .. " MyDirection " .. MyDirection);
			--print("Opposite Direction " .. OppDirection);
			--print("Detector = " .. DETECTOR .. " Reader = " .. READER);		
			--print(" Detector = " .. Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR));
			--print(" Reader = " ..   Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER));
			ExpectedReader = Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR) * 10 + Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER);
			print("Expected Reader Changed to " .. ExpectedReader);
			--
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (not GotBlock) then
				-- wait for MyNextBlockNo to be locked (either Yard track)
				local result;
				-- set occupancy flag in MyNextBlockNo
				SetOccupancy(MyNextBlockNo, MyEngineNo);	-- set occupancy flag in MyNextBlockNo
				-- set NextSwitch switch
				if (NextSwitch ~= nil) then
					if (MyNextBlockNo == Yard1) then
						Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("ML2Y : Switch " .. SwitchName[NextSwitch] .. " to NORMAL");
						end;
					if (MyNextBlockNo == Yard2) then
						Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("ML2Y : Switch " .. SwitchName[NextSwitch] .. " to REVERSE");
						end;
					end;
				GotBlock = true;	-- indicate needed Block is locked					
				end;
			if (EngineStopped) then
				-- start moving if stopped 
				local Aspect = GetCabAspect(HOMESIGNAL);
				print ("ML2Y : Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
---[[--TODO 3 -----------------------------------------------------------------------------------------------
				-- NOTE: MainLine2Yard can only be a permissive signal
				if (Signals and Aspect == AsStopSignal) then
					return EmergencyStop("ML2Y : home signal is an unexpected absolute signal");
					end;				
-----------------------------------------------------------------------------------------------------]]
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 5;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	-- UsedUpTime always <= StopDelay
					end;
				--print("ML2Y : Used Up Time ", UsedUpTime, " Stop Delay ", StopDelay, " Start Delay ", StartDelay);
				LastBlockSpeedLimit = BlockSpeedLimit;
				LastSentSpeed = SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				print(string.format("ML2Y : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));		
				EngineStopped = false;	-- indicate that we handled the stop here
				end;
				
			WaitforCaboose = false;
			end;
	return true;
	end

return MainLine2Yard;
end;

--[[---------------------------------------------------------------------------------------]]
function createStopInYard(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance or 24;		
	local EngineStopped = false;
	local StopDelay = 0;
	local StopTime = 0;	
	local Target_Distance;	
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;	
	local TrainTime = 0;	
	local function StopInYard(DetectorID, EngineNo, TagLocation, CarModel,  OffLayoutBlock)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (type(OffLayoutBlock) ~= "table") then
			return EmergencyStop("SIY : OffLayoutBlock not a table");
			end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			
--[[--- Engine Detected -----]]
			
			TrainTime = RunTime();	-- Engine detected time	
			print("SIY : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			local Spd = GetSpeed(MyEngineNo, MyTIUNo);	-- get speed as engine covers tag
			if (Spd ~= BlockSpeedLimit) then
				return EmergencyStop("SIY : Speed over tag = " .. Spd .. " Smph - expected = " .. BlockSpeedLimit .. " Smph");
				end;
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			-- initialize block local variables
			EngineStopped = false;
			GotBlock = false;
			--	
			-- Adjust speed for the current block
			LastBlockSpeedLimit = BlockSpeedLimit;
			BlockSpeedLimit = AdjustSpeed(CurrentBlockNo, AsNone);	-- adjust speed for some randomness in high speed blocks
			SetCounter(C3, BlockSpeedLimit);
			LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			print(string.format("SIY : %s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));	
			--
			NextBlockNo = OffLayoutBlock[1];
			
			-- schedule stop in CurrentBlockNo
			StopDelay, Target_Distance = StopAfterDelay(CurrentBlockNo, MyEngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
			EngineStopped = true;
			StopTime = RunTime();
			
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			DisplayBlockNames("SIY : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;
			
--[[--- Caboose Detected -----]]

		elseif (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION) and WaitforCaboose) then
			print("SIY : " .. DetectorID .. "End of Train after " .. RunTime() - TrainTime .. " seconds (Stop Delay = " .. StopDelay .. ")");
			if (SwitchClearingDistance > 0) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
				print(string.format("SIY : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], SwitchClearingDistance, BlockSpeedLimit, SleepDelay));
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				print("SIY : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				end;
			--
			--print("MyCurrentBlockNo " .. MyCurrentBlockNo .. " MyDirection " .. MyDirection);
			--print("Opposite Direction " .. OppDirection);
			--print("Detector = " .. DETECTOR .. " Reader = " .. READER);		
			--print(" Detector = " .. Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR));
			--print(" Reader = " ..   Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER));
			ExpectedReader = Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, DETECTOR) * 10 + Layout4.STOPBlock(MyCurrentBlockNo, OppDirection, READER);
			print("Expected Reader Changed to " .. ExpectedReader);
			--				
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (EngineStopped) then
				print("SIY : " .. DetectorID .. "Stopped!");
				RFIDFlag = false;
				end
			WaitforCaboose = false;
			end;	
	return true;
	end;

return StopInYard;
end;

--[[---------------------------------------------------------------------------------------]]

function setup(Engine, TIU)
print("setup() Running");

-- Instrumented Layout 4
Layout4 = require([[Layout4]]);		-- All of the details about the Layout
-- an undirected weighted double vertex graph
MyTIUNo = Layout4.MASTER_TIU;
MyEngineNo = Engine;	-- this engine number may be changed by the query below
Title(title, MyEngineNo);
print("Layout : " .. Layout4.name);

print([[Run "Layout APB Master.lua" to handle the signals]]);

math.randomseed(os.time());	-- random seed

-- this is done in Layout4 APB Master
--ClearAllOccupancy();

ShowLayout();
--[[--------------------------------------------]]
local EngineNameList = {};	-- table to hold all of the Engine numbers and Names
local EngineNameIndex = 1;
local EngineNameCount = 1;

for iEng = 1, 99 do	-- check all possible engines on one TIU
	if (Exists(iEng)) then
		local Name, PS;
		Name, PS = GetName(iEng);
		--print(iEng .. " " .. Name .. " PS" .. PS);
		EngineNameList[iEng] = iEng .. " " .. Name .. " PS" .. PS;
		if (MyEngineNo == iEng) then
			EngineNameIndex = EngineNameCount;
			--print("EngineNameIndex = " .. EngineNameIndex);
			end;
		EngineNameCount = EngineNameCount + 1;		
		end
	end
--
-- Ask the dispatcher for the engine number
--
print("Select Engine");
local Enum = InputDropDown(title, "Select Engine", EngineNameIndex, EngineNameList);
--print("Enum ", Enum);
local typ = type(Enum);
--print("Type ", typ);

if (typ == "boolean" and Enum == false) then
	return Stop("Canceled");
	end;
		
if (Enum ~= nil and Enum ~= "") then	-- if the result is a string, then use first number in the string, otherwise, ignore it
	-- pull first integer found in the string
	local EngNo = string.match(Enum, "%d+");
	if (EngNo ~= nil) then
		MyEngineNo = tonumber(EngNo);
		else
		return Stop("Invalid Engine Number");
		end;
	else
	if (typ == "nil") then
		-- nil returned only if parameters are invalid
		return Stop("Invalid Parameters to InputDropDown");
		else
		MyEngineNo = 1;
		end
	end	
--[[--------------------------------------------]]	
print("Setup : Engine Number = " .. MyEngineNo);

-- Show new title with Engine # and Name and put Engine # into Engine Number Spinner
local Name = GetName(MyEngineNo);
Title(title .. " : Engine #" .. MyEngineNo .. " " .. Name, MyEngineNo);	
--
-- start of run
--
--[[--------------------------------------------]]
--[[
-- Ask the dispatcher for direction - not needed for dedicated EB/WB scripts
--	
print("Select Direction");
local DirValue = InputDropDown(title, "Select Direction", MyDirection, {"1 Westbound", "2 Eastbound", "3 Northbound", "4 Southbound"});
--print("DirValue ", DirValue);
typ = type(DirValue);
--print("Type ", typ);

if (typ == "boolean" and DirValue == false) then
	return Stop("Canceled");
	end;
		
if (DirValue ~= nil and DirValue ~= "") then	-- if the result is a string, then use first number in the string, otherwise, ignore it
	-- pull first integer found in the string
	local DirNo = string.match(DirValue, "%d+");
	if (DirNo ~= nil) then
		MyDirection = tonumber(DirNo);
		else
		return Stop("Invalid Direction");
		end;
	else
	if (typ == "nil") then
		-- nil returned only if parameters are invalid
		return Stop("Invalid Parameters to InputDropDown");
		else
		--MyDirection is not changed
		end
	end	
]]
--[[--------------------------------------------]]	
print("Setup : Direction = " .. DirectionNames[MyDirection]);
--
--[[--------------------------------------------]]
	--
	-- Ask the dispatcher for starting block number
	--
	-- For westbound trains, starting block can be any Thru block or any mainline block
	-- For eastbound trains, starting block can be any Station/Passing block or any mainline block
	--
	local BlockNameList = {};	-- table to hold all of the block numbers and names

	for BlockNameIndex, Name in pairs(BlockName) do
		if (BlockNameIndex <= 100) then
			--print(Name .. "(" .. BlockNameIndex .. ")");
			if (MyDirection == EB) then
				-- don't allow selection of thru blocks
				if ((BlockNameIndex ~= J_L_THRU_TRACK) and (BlockNameIndex ~= COLLEGE_THRU_TRACK) and (BlockNameIndex ~= STRUTHERS_THRU_TRACK)) then 
					BlockNameList[BlockNameIndex] = BlockNameIndex .. " " .. Name; 
					end;
				end;
			if (MyDirection == WB) then
				-- don't allow selection of station/passing blocks
				if ((BlockNameIndex ~= J_L_PASSING_TRACK) and (BlockNameIndex ~= COLLEGE_STATION_TRACK) and (BlockNameIndex ~= STRUTHERS_STATION_TRACK)) then
					BlockNameList[BlockNameIndex] = BlockNameIndex .. " " .. Name; 
					end;
				end;
			end;
		end;
--
-- read Starting Block Number
print("Select Starting Block");
--
local StartingBlock = InputDropDown(title, "Select Starting Block", YOUNGSTOWN_BLOCK, BlockNameList);
--print("StartingBlock ", StartingBlock);
typ = type(StartingBlock);
--print("Type ", typ);

if (typ == "boolean" and StartingBlock == false) then
	return Stop("Canceled");
	end;
		
if (StartingBlock ~= nil and StartingBlock ~= "") then	-- if the result is a string, then use first number in the string, otherwise, ignore it
	-- pull first integer found in the string
	local BlockNo = string.match(StartingBlock, "%d+");
	if (BlockNo ~= nil) then
		StartBlockNo = tonumber(BlockNo);
		else
		return Stop("Invalid Block Number");
		end;
	else
	if (typ == "nil") then
		-- nil returned only if parameters are invalid
		return Stop("Invalid Parameters to InputDropDown");
		else
		StartBlockNo = 1;
		end
	end	
--[[--------------------------------------------]]
print("Setup : Starting Block = " .. BlockName[StartBlockNo]);	
CurrentBlockNo = StartBlockNo;
if (MyDirection == WB) then
	EndBlockNo = YOUNGSTOWN_YARD1_BLOCK;	-- EndBlockNo is not used because this sketch loops forever
	else
	EndBlockNo = MCKEES_ROCKS_YARD1_BLOCK;	-- EndBlockNo is not used because this sketch loops forever
	end
-- set the next expected detector and reader
--print("CurrentBlockNo " .. CurrentBlockNo .. " MyDirection " .. MyDirection);
--print("Opposite Direction " .. OppDirection);
--print("Detector = " .. DETECTOR .. " Reader = " .. READER);	
--print(" Detector = " .. Layout4.STOPBlock(CurrentBlockNo, OppDirection, DETECTOR));
--print(" Reader = " .. Layout4.STOPBlock(CurrentBlockNo, OppDirection, READER));
ExpectedReader = Layout4.STOPBlock(CurrentBlockNo, OppDirection, DETECTOR) * 10 + Layout4.STOPBlock(CurrentBlockNo, OppDirection, READER)
print("Expected Reader = " .. ExpectedReader);
-----------------------------------------------------------------------------------------------------
-- create and register the cab signals
print("Setup : Creating Cab Signals");
-- Register the new block's home and distant signals
local NumLightsHome, NumLightsDistant;
local HomeSignal = NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL];	-- next signal number
_, NumLightsHome = GetAspect(HomeSignal);	-- 1, 2 or 3 lights on the signal head
local DistantSignal = NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]
_, NumLightsDistant = GetAspect(DistantSignal);
CreateCab(NumLightsHome, NumLightsDistant);	-- create light signals - (home, distant)			
print("Setup : Registering the Cab Signals #", HomeSignal, " and #", DistantSignal);
RegisterCabSignals(HomeSignal, DistantSignal);
						-- associate the Home cab signal with next signal and
						-- the Distant cab signal with second signal.
						-- Signals must be created by the "Layout4 Signal Master.lua"
						-- script which should be run in a different PC window
print("Setup : Reflecting signals from Layout window into the Cab Display");
print("Setup : Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[(GetCabAspect(HOMESIGNAL))]);
print("Setup : Distant Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[(GetCabAspect(DISTANTSIGNAL))]);

print("Setup : press [Initialize Layout] then [Start Run] to begin");
-----------------------------------------------------------------------------------------------------	
-- 
BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
LastBlockSpeedLimit = BlockSpeedLimit;
SetCounter(C3, BlockSpeedLimit);
--
-- Set Loop Switch
if (LLLoop) then
	Switch(REVERSE, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0)
	else
	Switch(NORMAL, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0)
	end;
--[[-------------------------]]
if (MyDirection == EB) then
	print("Setup : Eastbound");
	else
	print("Setup : Westbound");
	end
--[[-------------------------]]
-- read up current sound volume
EngineSoundSave = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
--
OverrideOccupancy(CurrentBlockNo, MyEngineNo);	-- force set occupancy flag on the starting block
-- Stop the Garbage collector since this is a real-time program
-- Might also be possible to stop and restart it at the right time
-- Maybe stop it when the selected engine tag is encountered and restart
-- it when the selected caboose tag is encountered
collectgarbage("stop")
print("Setup : Garbage Collection stopped");
--collectgarbage("collect")
--collectgarbage("restart")
--
return true;		-- false=setup failed, true=setup succeeded
end
--[[---------------------------------------------------------------------------------------]]

function DisplayBlockNames(Title, LastBlockNo, CurrentBlockNo, NextBlockNo)
print(Title .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
end;
--[[---------------------------------------------------------------------------------------]]
function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
-- This function is called each time a tag programmed with 32 bytes of data is detected (TAGDATABLOCK4)
-- TagPacket contains 8 characters (4 hex digits) of UID and 32 characters
-- of block 4 data from the programmed tag.

BumpCounter(COUNTER01);	
local PacketLen = #TagPacket;

if (Debug() >= 8) then print("Packet Length = ", PacketLen); end;
if (Debug() >= 7) then print(TagPacket); end;

-- index the table Tags[] by tag ID, value is the tag packet itself
Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
TagCount = TagCount + 1;	-- count the number of tags

local MyReader = (Detector * 10) + Reader;

if (EngineNo > 0) then	-- it's an engine
	if (EngineNo == MyEngineNo) then
		if (BeepOn) then Beep(ASTERISK); end;
		-- next line shows a different way to quote a string using brackets
		printc(clGreen, string.format([[tag(%.2f) : Detector %d Reader %d Eng#%d %s %s #%s %s]],
			RunTime(), Detector, Reader, EngineNo, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber, GetTagLocation(TagLocation)));
		if (MyReader ~= ExpectedReader) then
			return EmergencyStop("Got Reader = " .. MyReader .. " Expected Reader = " .. ExpectedReader);
			end
		else
		-- ignore engines that are not me
		return true;	-- true=continue to process tags, false=stop processing tags
		end;
	-- Continue on to process the detected engine
	else	-- it's a car
	if (CarModel == CABOOSE or CarModel == OBSERVATION) then
		if (BeepOn) then Beep(EXCLAMATION); end;
		printc(clBlue, string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		else
		-- ignore all non-end of train cars
		return true;	-- true=continue to process tags, false=stop processing tags
		end;		
	-- Continue on to process end of train tags
	end

-- The complete packet received is in TagPacket 
-- (all of the fields have already been extracted into the first parameters to tag()
--   First 8 digits are the 4 byte Tag UID in hexidecimal
--   Next 32 digits are the 16 bytes of Block 4 Information read from the Tag in hexidecimal
if (Debug() >= 6) then print(string.format("Packet  %s  %s", string.sub(TagPacket, 1, 8),
	string.sub(TagPacket,9,40) )); end
--[[--------------------------]]
-- Westbound trains are superior
--[[--------------------------]]
--
-- ResetinProgress probably does not work anymore
--
if (ResetinProgress and MyDirection == WB) then
	-- run back to MCKEES_ROCKS_YARD_TAG
	if (Detector == MCKEES_ROCKS_YARD_DETECTOR and Reader == MCKEES_ROCKS_YARD_READER) then
		local DetectorID = "@ MCKEES_ROCKS_YARD_TAG ";
		local StopDelay = 0;
		if (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION)) then		-- going in reverse, execute on caboose detection
			print(DetectorID .. "End of Train detected");
			-- schedule stop in StartBlockNo
			StopDelay = StopAfterDelay(StartBlockNo, EngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
			SetDirection(FORWARD, StopDelay + 5, EngineNo, MyTIUNo);
			ResetinProgress = false;
			end;
		end;
	return true;

elseif (ResetinProgress and MyDirection == EB) then
	-- run back to YOUNGSTOWN_YARD_TAG
	if (Detector == YOUNGSTOWN_YARD_DETECTOR and Reader == YOUNGSTOWN_YARD_READER) then
		local DetectorID = "@ YOUNGSTOWN_YARD_TAG ";
		local StopDelay = 0;
		if (EngineNo == 0 and (CarModel == CABOOSE or CarModel == OBSERVATION)) then		-- going in reverse, execute on caboose detection
			print(DetectorID .. "End of Train detected");
			-- schedule stop in StartBlockNo
			StopDelay = StopAfterDelay(StartBlockNo, EngineNo, MyDirection, LastBlockSpeedLimit, BlockSpeedLimit)
			SetDirection(FORWARD, StopDelay + 5, EngineNo, MyTIUNo);
			ResetinProgress = false;
			end;
		end;
	return true;

elseif (not RFIDFlag) then	-- ignore tags unless engine is moving
	return true;
	
-- YOUNGSTOWN_YARD_TAG
elseif (Detector == YOUNGSTOWN_YARD_DETECTOR and Reader == YOUNGSTOWN_YARD_READER) then
	if (Func50 == nil) then
		Func50 = createMainLine2Passing(false);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock
	Func50("@ YOUNGSTOWN_YARD_TAG ", EngineNo, TagLocation, CarModel, STRUTHERS_THRU_TRACK, STRUTHERS_STATION_TRACK, nil);

-- STRUTHERS_WEST_TAG
elseif (Detector == STRUTHERS_WEST_DETECTOR and Reader == STRUTHERS_WEST_READER) then
	if (Func40 == nil) then
		Func40 = createPassing2MainLine(false, SwitchClearingInches[STRUTHERS_WEST_TAG][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set. Set STRUTHERS_WEST to NORMAL on Caboose detection.
	Func40("@ STRUTHERS_WEST_TAG ", EngineNo, TagLocation, CarModel, STRUTHERS_THRU_TRACK, STRUTHERS_STATION_TRACK, STRUTHERS_WEST, STRUTHERS_EAST, 
		{LOWELLVILLE_BLOCK, NEW_CASTLE_BLOCK, COLLEGE_STATION_TRACK});

-- STRUTHERS_EAST_TAG
elseif (Detector == STRUTHERS_EAST_DETECTOR and Reader == STRUTHERS_EAST_READER) then
	if (Func20 == nil) then
		Func20 = createMainLine2MainLine(false);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock. Set STRUTHERS_EAST to NORMAL on Caboose detection.	
	Func20("@ STRUTHERS_EAST_TAG ", EngineNo, TagLocation, CarModel, STRUTHERS_EAST,
		{NEW_CASTLE_BLOCK, COLLEGE_STATION_TRACK});

-- LOWELLVILLE_TAG
elseif (Detector == LOWELLVILLE_DETECTOR and Reader == LOWELLVILLE_READER) then		-- Split New Castle Block
	if (Func90 == nil) then
		Func90 = createMainLine2Passing(false);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock
	Func90("@ LOWELLVILLE_TAG ", EngineNo, TagLocation, CarModel, COLLEGE_THRU_TRACK, COLLEGE_STATION_TRACK, nil);

-- COLLEGE_WEST_TAG
elseif (Detector == COLLEGE_WEST_DETECTOR and Reader == COLLEGE_WEST_READER) then
	if (Func21 == nil) then
		Func21 = createPassing2MainLine(false, SwitchClearingInches[COLLEGE_WEST_TAG][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set. Set COLLEGE_WEST to NORMAL on Caboose detection.
	Func21("@ COLLEGE_WEST_TAG ", EngineNo, TagLocation, CarModel, COLLEGE_THRU_TRACK, COLLEGE_STATION_TRACK, COLLEGE_WEST, COLLEGE_EAST, 
		{BEAVER_FALLS_BLOCK, FALLSTON_BLOCK, MONACA_BLOCK, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK});

-- COLLEGE_EAST_TAG
elseif (Detector == COLLEGE_EAST_DETECTOR and Reader == COLLEGE_EAST_READER) then
	if (Func41 == nil) then
		Func41 = createMainLine2MainLine(false); 
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	.  Set COLLEGE_EAST to NORMAL on Caboose detection.
	Func41("@ COLLEGE_EAST_TAG ", EngineNo, TagLocation, CarModel, COLLEGE_EAST,
		{ FALLSTON_BLOCK, MONACA_BLOCK, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK,	J_L_PASSING_TRACK});

-- FALLSTON_TAG
elseif (Detector == FALLSTON_DETECTOR and Reader == FALLSTON_READER) then		-- Split Beaver Falls Block
	if (Func101 == nil) then
		Func101 = createMainLine2MainLine(false);
		end;
	Func101("@ FALLSTON_TAG ", EngineNo, TagLocation, CarModel, nil,
		{MONACA_BLOCK, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK});

-- MONACA_WEST_TAG
elseif (Detector == MONACA_WEST_DETECTOR and Reader == MONACA_WEST_READER) then
	-- entering tunnel, reduce volume
	if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK and SoundOff == false) then
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 0, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 1, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 1/2), 2, MyEngineNo, MyTIUNo);
		end;
	if (Func61 == nil) then
		Func61 = createMainLine2MainLine(false);
		end;
	Func61("@ MONACA_WEST_TAG ", EngineNo, TagLocation, CarModel, nil,
		{WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK});

-- MONACA_EAST_TAG
elseif (Detector == MONACA_EAST_DETECTOR and Reader == MONACA_EAST_READER) then
	-- leaving tunnel, increase volume
	if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK and SoundOff == false) then
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 0, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 1, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, EngineSoundSave, 2, MyEngineNo, MyTIUNo);
		end;
	if (Func60 == nil) then
		Func60 = createMainLine2MainLine(false);
		end;
	
	Func60("@ MONACA_EAST_TAG ", EngineNo, TagLocation, CarModel, nil,
		{ALIQUIPPA_BLOCK,  J_L_PASSING_TRACK});

-- WEST_ALIQUIPPA_TAG
elseif (Detector == WEST_ALIQUIPPA_DETECTOR and Reader == WEST_ALIQUIPPA_READER) then		-- Split Aliquippa Block
	if (Func100 == nil) then
		Func100 = createMainLine2Passing(false);
		end;
	Func100("@ WEST_ALIQUIPPA_TAG ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, nil);

elseif (Detector == ALIQUIPPA_WEST_DETECTOR and Reader == ALIQUIPPA_WEST_READER) then
	if (LLLoop == false) then
		if (Func10 == nil) then
			Func10 = createPassing2MainLine(false, SwitchClearingInches[ALIQUIPPA_WEST_TAG][MyDirection]);
			end;
		Func10("@ 1-0 ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, ALIQUIPPA_WEST, ALIQUIPPA_EAST,
			{MCKEES_ROCKS_BLOCK, EndBlockNo});
		else
		if (Func10 == nil) then
			Func10 = createPassing2MainLine(false, SwitchClearingInches[ALIQUIPPA_WEST_TAG][MyDirection]);
			end;
		-- status: in block NextBlockNo, occupancy already set. Set ALIQUIPPA_WEST to NORMAL on Caboose detection.
		Func10("@ ALIQUIPPA_WEST_TAG ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, ALIQUIPPA_WEST, ALIQUIPPA_EAST,
			{MCKEES_ROCKS_BLOCK, LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK});
		end;

-- ALIQUIPPA_EAST_TAG
elseif (Detector == ALIQUIPPA_EAST_DETECTOR and Reader == ALIQUIPPA_EAST_READER) then
	-- status: just entered NextBlockNo, occupancy already set
	if (LLLoop == false) then
		if (Func51 == nil) then
			Func51 = createMainLine2Yard(false, SwitchClearingInches[ALIQUIPPA_EAST_TAG][MyDirection]);
			end;
		Func51("@ ALIQUIPPA_EAST_TAG ", EngineNo, TagLocation, CarModel, MCKEES_ROCKS_YARD1_BLOCK, MCKEES_ROCKS_YARD2_BLOCK, MCKEES_ROCKS_YARD,
			{EndBlockNo});
		else
		if (Func51 == nil) then
			Func51 = createMainLine2MainLine(false);
			end;
		if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then
			Switch(REVERSE, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);
			end;
		-- engine never stops in this block except for deadlock.  Set ALIQUIPPA_EAST to NORMAL on Caboose detection.
		Func51("@ ALIQUIPPA_EAST_TAG ", EngineNo, TagLocation, CarModel, ALIQUIPPA_EAST,
			{LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK});
		end;

-- MCKEES_ROCKS_YARD_TAG
elseif (Detector == MCKEES_ROCKS_YARD_DETECTOR and Reader == MCKEES_ROCKS_YARD_READER) then
	-- status: in block NextBlockNo, occupancy already set
	if (LLLoop == false) then
		if (Func30 == nil) then
			Func30 = createStopInYard(false, SwitchClearingInches[MCKEES_ROCKS_YARD_TAG][MyDirection]);	-- clear yard switch
			end;
		Func30("@ MCKEES_ROCKS_YARD_TAG ", EngineNo, TagLocation, CarModel, {POINTS_SOUTH_N_W});
		else	
		if (Func30 == nil) then
			Func30 = createMainLine2MainLine(false);	-- no switches if staying on Mainline
			end;
		-- engine never stops in this block except for deadlock
		Func30("@ MCKEES_ROCKS_YARD_TAG ", EngineNo, TagLocation, CarModel, nil,
			{YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK});
		end;
	end;

return true;	-- true=continue to process tags, false=stop processing tags
end	
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
-- this function is called once when the user presses the [STOP] button or the Stop() function is called
print("All Newly Detected Tags:");
local k, v;
for k,v in  pairs(Tags) do		-- dump out the Tags table
	--                           . . . . . . . . . . . . . . . .
	-- print format:  6619044A  88080101030108011B02202031353536
	print(k .. "  " .. v);
	end
LastSentSpeed = SetSpeed(0, 0, MyEngineNo, MyTIUNo);
Rate(Layout4.AccDecRate, Layout4.AccDecRate, 2, MyEngineNo, MyTIUNo);
print("Shutdown in Progress.....");
--
if (StartedFlag) then
	print("Shutting down the Engine");
	Sleep(10);
	EngineShutdown(MyEngineNo, MyTIUNo);
	StartedFlag = false;
	end
RFIDFlag = false;
ResetinProgress = false;
print(string.format("cleanup() Complete at %.2f", RunTime()));
return true;		-- false=cleanup failed, true=cleanup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function Plus5Button(Eng, TIU)
BlockSpeedLimit = BlockSpeedLimit + 5;
LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
SetCounter(C3, BlockSpeedLimit)
return true;
end
Function01Name, Function01Label = Plus5Button, "Speed +5";
--[[---------------------------------------------------------------------------------------]]
function Minus5Button(Eng, TIU)
BlockSpeedLimit = BlockSpeedLimit - 5;
LastSentSpeed = SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
SetCounter(C3, BlockSpeedLimit)
return true;
end
Function02Name, Function02Label = Minus5Button, "Speed -5";
--[[---------------------------------------------------------------------------------------]]
function DTOButton(Eng, TIU)
print(string.format("Engine %d Trip Odometer %.3f Smiles", MyEngineNo, DTO(MyEngineNo, MyTIUNo)));
return true;
end
Function21Name, Function21Label = DTOButton, "Show DTO";
--[[---------------------------------------------------------------------------------------]]
function DODButton(Eng, TIU)
print(string.format("Engine %d Odometer %.3f Smiles", MyEngineNo, DOD(MyEngineNo, MyTIUNo)));
return true;
end
Function22Name, Function22Label = DODButton, "Show DOD";
--[[---------------------------------------------------------------------------------------]]
function ClearButton(Eng, TIU)
print("Clear Occupancy Flags");
-- clear occupancy flags on request but not at startup (cleared in "Layout4 APB Master")
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, UNOCCUPIED);
	end;
print("Occupancy Flags cleared");
return true;
end
Function03Name, Function03Label = ClearButton, "Clear Flags";
--[[---------------------------------------------------------------------------------------]]
function HeadwayButton(Eng, TIU)
print("Toggle Headway");
-- toggle Headway flag
Headway =  not Headway;
print("Headway set to " .. tostring(Headway));
return true;
end
Function04Name, Function04Label = HeadwayButton, "Headway";
--[[---------------------------------------------------------------------------------------]]
function SignalsButton(Eng, TIU)
print("Toggle Signals");
-- toggle Signals flag
Signals =  not Signals;
print("Signals set to " .. tostring(Signals));
return true;
end
Function05Name, Function05Label = SignalsButton, "Signals";
--[[---------------------------------------------------------------------------------------]]
function SoundOffButton(Eng, TIU)
print("Sound Off");
if (SoundOff == false) then
	-- read up current sound volume
	EngineSoundSave = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
	print(" Sound Off - Engine " .. MyEngineNo .. " Level was " .. EngineSoundSave);
	SoundOff = true;
	SetVolume(MASTER_VOLUME, 0, 3, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function19Name, Function19Label = SoundOffButton, "Sound Off";
--[[---------------------------------------------------------------------------------------]]
function SoundOnButton(Eng, TIU)
print("Sound On");
if (SoundOff == true) then
	print(" Sound On - Engine " .. MyEngineNo .. " Level restored to " .. EngineSoundSave);
	SoundOff = false;
	SetVolume(MASTER_VOLUME, EngineSoundSave, 3, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function20Name, Function20Label = SoundOnButton, "Sound On";
--[[---------------------------------------------------------------------------------------]]
function InitButton(Eng, TIU)
print("Initialize");
print("Set all Switches to NORMAL postion");
local k, v;
-- set all switches to NORMAL position
for k,v in pairs(Switches) do
	print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
	Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
	Sleep(0.5);
	end
if (Debug() > 3) then
	-- set all switches to REVERSE
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(REVERSE, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	-- set all switches to NORMAL
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	end;
-- do not clear occupancy flags - its done in Layout4 APB Master
--[[
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, UNOCCUPIED);
	end;
]]
--[[-----------------------------]]
-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func90 = nil;	-- Split New Castle Block
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;
Func100 = nil;	-- Split Alquippa block
Func101 = nil;	-- Split Beaver Falls Block

print("Initialization is Complete");
return true;
end
Function06Name, Function06Label = InitButton, "Initialize Layout";
--[[---------------------------------------------------------------------------------------]]
function StartButton(Eng, TIU)
print("Start the Run");

--[[-----------------------------]]
-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func90 = nil;	-- Split New Castle Block
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;
Func100 = nil;	-- Split Alquippa block
Func101 = nil;	-- Split Beaver Falls Block

--[[------------------------------------------]]
--print("StationName = " .. Pretty(StationName));
--print("StationName = " .. Pretty(BlockName));
--print("Switches = " ..  Pretty(Switches));
--print("NextBlock = " ..  Pretty(NextBlock));
print("Headway set to " .. tostring(Headway));
print("Signals set to " .. tostring(Signals));
-- start the engine
Rate(Layout4.AccDecRate, Layout4.AccDecRate, 0, MyEngineNo, MyTIUNo);
print("setup : Set accell/decell rates to " .. Layout4.AccDecRate);
if (StartedFlag == false) then
	EngineStartUp(MyEngineNo, MyTIUNo);
	print("Starting up the Engine");
	StartedFlag = true;
	Sleep(20);
	end
SetDirection(FORWARD, 0, MyEngineNo, MyTIUNo);
OverrideOccupancy(CurrentBlockNo, MyEngineNo);	-- force set occupancy flag for starting block
--
-- Engine can start in any block, set exit switches from meet points as necessary
--
if (CurrentBlockNo == YOUNGSTOWN_YARD1_BLOCK and MyDirection == EB) then
	LastBlockNo = ASHTABULA_NYC;
	NextBlockNo = YOUNGSTOWN_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, STRUTHERS_STATION_TRACK}, MyEngineNo);
	-- position YOUNGSTOWN_YARD switch as needed depending on value of CurrentBlockNo
	Switch(NORMAL, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	-- position BALLOON switch as needed after locking next block
	LLLoop = false;
	Switch(NORMAL, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);	
	
elseif (CurrentBlockNo == YOUNGSTOWN_YARD2_BLOCK and MyDirection == EB) then
	LastBlockNo = CLEVELAND_EL;
	NextBlockNo = YOUNGSTOWN_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, STRUTHERS_STATION_TRACK}, MyEngineNo);		
	-- position YOUNGSTOWN_YARD switch as needed depending on value of CurrentBlockNo
	Switch(REVERSE, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	-- position BALLOON switch as needed after locking next block
	LLLoop = false;
	Switch(NORMAL, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);	
	
elseif (CurrentBlockNo == YOUNGSTOWN_BLOCK and MyDirection == EB) then -- "Youngstown Block"
	if (LLLoop == true) then
		LastBlockNo = LIONEL_IVES_BLOCK;
		NextBlockNo = STRUTHERS_STATION_TRACK;
		else
		LastBlockNo = StartBlockNo;
		NextBlockNo = STRUTHERS_STATION_TRACK;
		end;
	-- check that the Next Block is not occupied
	SingleSetOccupancy({NextBlockNo}, MyEngineNo);

elseif (CurrentBlockNo == STRUTHERS_STATION_TRACK and MyDirection == EB) then -- "Struthers Station Track"
	LastBlockNo = YOUNGSTOWN_BLOCK;
	NextBlockNo = LOWELLVILLE_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, NEW_CASTLE_BLOCK, COLLEGE_STATION_TRACK}, MyEngineNo);
	-- Position STRUTHERS_EAST after locking next block
	Switch(REVERSE, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0);	

elseif (CurrentBlockNo == STRUTHERS_THRU_TRACK and MyDirection == EB) then -- "Struthers Thru Track" -- Eastbound trains don't use the thru track
	LastBlockNo = YOUNGSTOWN_BLOCK;
	NextBlockNo = LOWELLVILLE_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, NEW_CASTLE_BLOCK, COLLEGE_STATION_TRACK}, MyEngineNo);			
	-- Position STRUTHERS_EAST after locking next block
	Switch(NORMAL, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0);	

elseif (CurrentBlockNo == LOWELLVILLE_BLOCK and MyDirection == EB) then -- "Lowellville Block"
	LastBlockNo = STRUTHERS_STATION_TRACK;
	NextBlockNo = NEW_CASTLE_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, COLLEGE_STATION_TRACK}, MyEngineNo);
	
elseif (CurrentBlockNo == NEW_CASTLE_BLOCK and MyDirection == EB) then -- "New Castle Block"
	LastBlockNo = LOWELLVILLE_BLOCK;
	NextBlockNo = COLLEGE_STATION_TRACK;
	-- check that the Next Block is not occupied
	SingleSetOccupancy({NextBlockNo}, MyEngineNo);				
		
elseif (CurrentBlockNo == COLLEGE_STATION_TRACK and MyDirection == EB) then -- "College Station Track"
	LastBlockNo = NEW_CASTLE_BLOCK;
	NextBlockNo = BEAVER_FALLS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, FALLSTON_BLOCK, MONACA_BLOCK, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);				
	-- Position COLLEGE_EAST after locking next block
	Switch(REVERSE, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0);
	
elseif (CurrentBlockNo == COLLEGE_THRU_TRACK and MyDirection == EB) then -- "College Thru Track" -- Eastbound trains don't use the thru track
	LastBlockNo = NEW_CASTLE_BLOCK;
	NextBlockNo = BEAVER_FALLS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, FALLSTON_BLOCK, MONACA_BLOCK, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);				
	-- Position COLLEGE_EAST after locking next block
	Switch(NORMAL, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0);
	
elseif (CurrentBlockNo == BEAVER_FALLS_BLOCK and MyDirection == EB) then -- "Beaver Falls Block"
	LastBlockNo = COLLEGE_STATION_TRACK;
	NextBlockNo = FALLSTON_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, MONACA_BLOCK, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);	

elseif (CurrentBlockNo == FALLSTON_BLOCK and MyDirection == EB) then -- "Fallston Block"
	LastBlockNo = BEAVER_FALLS_BLOCK;
	NextBlockNo = MONACA_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, WEST_ALIQUIPPA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);
	
elseif (CurrentBlockNo == MONACA_BLOCK and MyDirection == EB) then -- "Monaca Block"
	LastBlockNo = FALLSTON_BLOCK;
	NextBlockNo = WEST_ALIQUIPPA_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, ALIQUIPPA_BLOCK,  J_L_PASSING_TRACK}, MyEngineNo);

elseif (CurrentBlockNo == WEST_ALIQUIPPA_BLOCK and MyDirection == EB) then -- "West Aliquippa Block"
	LastBlockNo = MONACA_BLOCK;
	NextBlockNo = ALIQUIPPA_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, J_L_PASSING_TRACK}, MyEngineNo);

elseif (CurrentBlockNo == ALIQUIPPA_BLOCK and MyDirection == EB) then -- "Aliquippa Block"
	LastBlockNo = WEST_ALIQUIPPA_BLOCK;
	NextBlockNo = J_L_PASSING_TRACK;
	-- check that the Next Block is not occupied
	SingleSetOccupancy({NextBlockNo}, MyEngineNo);
	
elseif (CurrentBlockNo == J_L_PASSING_TRACK and MyDirection == EB) then -- "J&L Passing Track"
	LastBlockNo = ALIQUIPPA_BLOCK;
	NextBlockNo = MCKEES_ROCKS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK}, MyEngineNo);
	-- Position ALIQUIPPA_EAST after locking next block
	Switch(REVERSE, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0);

elseif (CurrentBlockNo == J_L_THRU_TRACK and MyDirection == EB) then -- "J&L Thru Track" -- Eastbound trains don't use the thru track
	LastBlockNo = ALIQUIPPA_BLOCK;
	NextBlockNo = MCKEES_ROCKS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK}, MyEngineNo);				
	-- Position ALIQUIPPA_EAST after locking next block
	Switch(NORMAL, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0);
	
elseif (CurrentBlockNo == MCKEES_ROCKS_BLOCK and MyDirection == EB) then -- "McKees Rocks Block"
	if (LLLoop == true) then
		LastBlockNo = J_L_PASSING_TRACK;
		NextBlockNo = LIONEL_IVES_BLOCK;
		else
		LastBlockNo = J_L_PASSING_TRACK;
		NextBlockNo = EndBlockNo;
		end;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK}, MyEngineNo);				
			
elseif (CurrentBlockNo == LIONEL_IVES_BLOCK and MyDirection == EB) then -- "Lionel-Ives Block" -- AKA Balloon Track
	LastBlockNo = MCKEES_ROCKS_BLOCK;
	NextBlockNo = YOUNGSTOWN_BLOCK;	-- check that the Next Block is not occupied
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, STRUTHERS_STATION_TRACK}, MyEngineNo);
	end;
--
DisplayBlockNames("Start Run : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
--
RFIDFlag = true;
Whistle(ON, 20, MyEngineNo, MyTIUNo);
Whistle(OFF, 22, MyEngineNo, MyTIUNo);
print("Trip Odometer " .. DTO(MyEngineNo, MyTIUNo)); 	
Sleep(30);

--
BlockSpeedLimit = BlockSpeed[MyDirection][StartBlockNo];
LastSentSpeed = SetSpeed(BlockSpeedLimit, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, BlockSpeedLimit);
LastBlockSpeedLimit = BlockSpeedLimit;
--	
--[[----------------------------------------------------]]
print(string.format("RunTime = %.2f seconds", RunTime()));
-- TODO once both engines exit the yard, throw the Balloon Switches
return true;
end
Function07Name, Function07Label = StartButton, "Start Run";
--[[---------------------------------------------------------------------------------------]]

function event(FromPC, Type, P1, P2, P3)
print("event() Running - From PC# ", FromPC, " EventType = ", Type, " Parameters = ", P1, " ", P2, " ", P3);
return true;		-- false=event failed, true=event succeeded	
end
--[[---------------------------------------------------------------------------------------]]

function OccButton(Eng, TIU)
printc(clGreen, "Occupancy Flags -------------------------");
local x, FColor;
for x = FIRST_BLOCK, LAST_BLOCK do
	local flag = GetFlag(x);
	if (flag == 0) then
		FColor = clBlack;
		else
		FColor = clRed;
		end;
	printc(FColor, BlockName[x] .. "(" .. x .. ") = ", flag)
	end;	
return true;
end
Function08Name, Function08Label = OccButton, "Occupancy Flags";
--[[---------------------------------------------------------------------------------------]]
-- Run the engine back to the start (in reverse)
function ResetButton()
if (ResetinProgress) then
	return;	-- already being reset
	end;
print("Return to Initial Position");
-- stop the engine
LastSentSpeed = SetSpeed(0, 0, MyEngineNo, MyTIUNo);
RFIDFlag = false;
Sleep(5);
--[[-----------------------------]]
-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func90 = nil;	-- Split New Castle Block
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;
Func100 = nil;	-- Split Alquippa block
Func101 = nil;	-- Split Beaver Falls Block

-- position yard switch as needed
-- depending on value of StartBlockNo
if (StartBlockNo == MCKEES_ROCKS_YARD1_BLOCK) then
	Switch(NORMAL, Switches[MCKEES_ROCKS_YARD][CHAN_NUMBER], Switches[MCKEES_ROCKS_YARD][AIU_NUMBER], Switches[MCKEES_ROCKS_YARD][TIU_NUMBER], 0);

elseif (StartBlockNo == MCKEES_ROCKS_YARD2_BLOCK) then
	Switch(REVERSE, Switches[MCKEES_ROCKS_YARD][CHAN_NUMBER], Switches[MCKEES_ROCKS_YARD][AIU_NUMBER], Switches[MCKEES_ROCKS_YARD][TIU_NUMBER], 0);

elseif (StartBlockNo == YOUNGSTOWN_YARD1_BLOCK) then
	Switch(NORMAL, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);

elseif (StartBlockNo == YOUNGSTOWN_YARD2_BLOCK) then
	Switch(REVERSE, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
--
-- start the engine
if (not StartedFlag) then
	print("Starting up the Engine");
	EngineStartUp(MyEngineNo, MyTIUNo);
	StartedFlag = true;
	Sleep(20);
	end
-- direction to reverse
SetDirection(REVERSE, 0, MyEngineNo, MyTIUNo);
-- 
BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
LastSentSpeed = SetSpeed(BlockSpeedLimit, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, BlockSpeedLimit);
LastBlockSpeedLimit = BlockSpeedLimit;
--
ResetinProgress = true;
return true
end;
Function12Name, Function12Label = ResetButton, "Initial Position";
--[[---------------------------------------------------------------------------------------]]

function ZeroSpeedButton()
	print("Speed set to 0 Smph, was " .. LastSentSpeed .. " Smph");
	SetSpeed(0, 0, MyEngineNo, MyTIUNo);	
	return true;
end
Function09Name, Function09Label = ZeroSpeedButton, "Set Speed = 0";
--[[---------------------------------------------------------------------------------------]]

function ResumeSpeedButton()
	-- resume speed at LastSentSpeed
	print("Resume Speed " .. LastSentSpeed .. " Smph");
	LastSentSpeed = SetSpeed(LastSentSpeed, 0, MyEngineNo, MyTIUNo);	
	return true;
end
Function10Name, Function10Label = ResumeSpeedButton, "Resume Speed";
--[[---------------------------------------------------------------------------------------]]

function ReloadButton()
	print("Reloading layout4.lua");
	package.loaded["layout4"] = nil;	-- unload
	Layout4 = require([[layout4]]);	-- All of the details about the Layout
	return true;
end
Function11Name, Function11Label = ReloadButton, "Reload";
--[[---------------------------------------------------------------------------------------]]

function BeepOffButton()
	print("Beep Off");
	BeepOn = false;
	return true;
end
Function14Name, Function14Label = BeepOffButton, "Beep Off";
--[[---------------------------------------------------------------------------------------]]

function BeepOnButton()
	print("Beep On");
	BeepOn = true;
	return true;
end
Function15Name, Function15Label = BeepOnButton, "Beep On";
--[[---------------------------------------------------------------------------------------]]
function LLLButton()
	if (LLLoop) then
		print("Balloon Loop Off");
		LLLoop = false;
		Switch(NORMAL, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);	
		else
		print("Balloon Loop On");
		LLLoop = true;
		Switch(REVERSE, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);	
		end;
	return true;
end
Function16Name, Function16Label = LLLButton, "Balloon Loop";
--[[---------------------------------------------------------------------------------------]]
-- Generate a block speed based on Signal Aspects with some randomness for Normal and Limited speeds
function AdjustSpeed(BlockNo, Aspect)
local MySpeed, SignaledSpeed;
	-- each block has a speed limit
	MySpeed = BlockSpeed[MyDirection][BlockNo];
	SignaledSpeed = NORMAL_SPEED;
---[[--TODO---------------------------------------------------------------------------------------------------
	if (Signals) then
		-- each block also has a speed limit based on the signal aspect
		if (Aspect == AsClear) then
			SignaledSpeed = NORMAL_SPEED;
			elseif (Aspect == AsApproachLimited or Aspect == AsLimitedClear or Aspect ==  AsLimitedApproach) then
				SignaledSpeed = LIMITED_SPEED;
				elseif (Aspect == 	AsApproachMedium or Aspect == AsMediumClear or Aspect == AsApproach or Aspect == AsMediumApproach) then
					SignaledSpeed = MEDIUM_SPEED; 
					elseif (Aspect == AsApproachSlow or Aspect == AsSlowClear or Aspect == AsSlowApproach) then
						SignaledSpeed = SLOW_SPEED; 
						-- a stop aspect does not require restricting speeds
						--elseif (Aspect == AsRestricting or Aspect == AsStopandProceed or Aspect == AsStopSignal) then
						elseif (Aspect == AsRestricting) then
							SignaledSpeed = RESTRICTED_SPEED;
							end;
		--print("MySpeed = " .. MySpeed .. " Smph - Signaled Speed = " .. SignaledSpeed .. " Smph");
		if (SignaledSpeed < MySpeed) then MySpeed = SignaledSpeed; end;
	end;
-----------------------------------------------------------------------------------------------------]]
--[[
	if (MySpeed >= 25) then	-- adjust speed for some randomness in high speed blocks
		if (math.random(1,5) == 1) then
			MySpeed = (MySpeed // 5 ) * 4;
			end;
		end;
		--]]
---[[--TODO---------------------------------------------------------------------------------------------------
	if (Signals) then
		print("MySpeed = " .. MySpeed .. " Smph - Signaled Speed = " .. SignaledSpeed .. " Smph");
		else
		print("MySpeed = " .. MySpeed .. " Smph");
		end;
-----------------------------------------------------------------------------------------------------]]
	return MySpeed;
end;
--[[---------------------------------------------------------------------------------------]]
function WaitforStopSignaltoClear(Wait)
local Aspect = (GetCabAspect(HOMESIGNAL));
-- check that home signal is AsStopSignal, if it is, we MUST Stop until it clears
if (Aspect == AsStopSignal) then
	-- we are stopped, so wait until the Stop Signal aspect is cleared.
	print("Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
	local WaitTime = Wait or 1800;	-- let's say that 30 minutes is 'forever'
	-- Wait for the signal to changed from Stop Signal
	while (Aspect == AsStopSignal) do
		-- wait 'forever'
		if (((WaitTime >> 1) % 10) == 0) then
			print("Waiting for Stop Signal to clear");
			end;
		Sleep(2);
		WaitTime = WaitTime - 2;
		if (WaitTime <= 0) then
			return EmergencyStop("Timed out waiting for Stop Signal to clear"); 
			end;
		Aspect = (GetCabAspect(HOMESIGNAL));
		end;
	print ("Home Signal cleared to : " .. AspectNames[Aspect]);
	end;
end;
--[[---------------------------------------------------------------------------------------]]