--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

 Copyright 2019 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]

-- Semicolons are not required in Lua code
title = "3 Engine Demo"

-- This script is written to use 3 engines. The Engine List on the Main window
-- should have exactly three active engines and the rest should be inactive.

require([[defines]]);
require([[functions]]);

MyTIUNo = 99;	-- Global TIU number

DETECTED = true;
NOTDETECTED = false;

STOPDELAY = 2;	-- delay from tag detection to engine speed 0 command

-- all tables in Lua are 1 based, that is, T[1] is the first item in the list
EngineDetectionOrder = {};	-- clear the tables
EnginesUsedNum = {};
EnginesUsedFlag = {};
EngineSpeedSave = {};
Got = {};
-- Counter Labels
Counter01Label = "Engines Detected";
Counter02Label = "State Machine";
-- table to hold all of the tags detected in sequence
Tags = {};
-- state machine states
IDLE = 0;
DETECTENGINES = 1;
READYGO_3 = 2;
READYGO_3a = 3;
READYGO_3b = 4;
SM = IDLE;
-- Speeds
STARTSPEED = 15;
FIRSTSPEED = 22;
BASESPEED = 20;

--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	-- this function is called once when the [Run Script] button is pushed
	print("setup()");
	-- Engine number selected on the Main window is not used
	MyTIUNo = TIU;		-- TIU number selected on the Main window
	-- debug level, values 0 (off) to 9 (maximum)
	if (Debug() > 0) then print(string.format("Debug level %d", Debug())); end
	if (Debug() > 0) then print("Debug level " .. tonumber(Debug())); end
	-- initialize variables
	TagCount = 0;
	iEngineCount = 0;
	iEngineDetectionCount = 0;
	iReqEngines = 3;	-- number of engines required for this program
	UniqueTagCount = 0;	-- number of unique tags found
	
	-- make sure that RFID is connected
	TIUSerialPortConnected, TIUConnected, RFIDSerialPortConnected = CommStatus();	
	--print("TIUSerialPortConnected : " .. tostring(TIUSerialPortConnected) .. " - TIUConnected : " .. tostring(TIUConnected) .. " - RFIDSerialPortConnected : " .. tostring(RFIDSerialPortConnected));
	if (not RFIDSerialPortConnected) then
		print("RFID Port must be enabled for this script");
		return false;	-- false=setup failed
		end

	math.randomseed(os.time());	-- random seed
	assert(true);	-- debug only	
	-- initialize the layout
	ActiveCount, InactiveCount = Counts(MyTIUNo);
	-- Require three engines to run this script
	if (ActiveCount == 3) then  
		print("Three Active Engines Listed");
		else 
		SM = IDLE;
		print("Three Active Engines Not Listed, stopping the script...");
		return false;	-- setup failed
		end
	-- Start All Active Engines - 1-99 (DCS 2-100)
	print("Starting All Active Engines");

	for iEng = 1, 99 do	-- check all possible engines on one TIU
		if (Exists(iEng)) then
			if (IsActive(iEng)) then
				print("Engine " .. iEng .. " Starting");
				EngineStartUp(iEng, MyTIUNo);
				Sleep(2);
				end
			end
			
		if (IsEngineRunning(iEng, MyTIUNo)) then	-- IsEngineRunning() REQUIRES use of StartUpActiveEngines() or EngineStartUp()
			print(string.format("Initalizing Engine %d", iEng));
			-- save the numbers of all started engines
			iEngineCount = iEngineCount + 1;			-- Count the number of engines running
			EnginesUsedNum[iEngineCount] = iEng;		-- EnginesUsedXXXX[] is 1 based
			EnginesUsedFlag[iEngineCount] = NOTDETECTED;-- flag as not yet detected in motion
	   		-- Numberboards
			Setting(CMD_NUMBERBOARDS_ON,  18, iEng, MyTIUNo);	-- not a dedicated switch
			-- Set smoke off
			Smoke(OFF, 2, iEng, MyTIUNo);
			-- Cab Chatter Off
			CabChat(OFF, 3, iEng, MyTIUNo);
			-- Set Marker lights on
			Markers(ON, 15, iEng, MyTIUNo);
			-- Set Acc/Dec rate to 1/1 Smph/sec
			Rate(1, 1, 0, iEng, MyTIUNo);
			end
		end
	print(string.format("Engines Started %d", iEngineCount));
	-- Start each train moving forward - go slow to help insure tag detection
	-- Since we don't know the position of each engine, we start each at exactly
	-- the same speed at exactly the same time, once we have detected the engines
	-- and located them on the layout, we can vary the movement times and running speeds.
	for iDex = 1, iEngineCount do	-- for each running engine, set the speed to STARTSPEED Smph
		local iEng = EnginesUsedNum[iDex];	-- EnginesUsedXXXX[] is 1 based
		print(string.format("SetSpeed for Engine : %d", iEng));
		SetSpeed(STARTSPEED, 15 + iDex, iEng, MyTIUNo);
		end
	-- after a while, check that engines are up to speed
	Sleep(40);
	for iDex = 1, iEngineCount do
		-- read up speed and check it against STARTSPEED
		local iEng = EnginesUsedNum[iDex];	-- EnginesUsedXXXX[] is 1 based
		InstantSpeed = GetSpeed(iEng, TIU);
		if (InstantSpeed ~= STARTSPEED) then
			print(string.format("Instantaneous Speed Wrong for Engine %d : %d Smph", iEng, InstantSpeed));
			SetLED(1, clRed);	-- set status LED to red
			return Stop("End of Three Engine Demo");
			else
			print(string.format("Instantaneous Speed Correct for Engine %d : %d Smph", iEng, InstantSpeed));
			end;
		end;
	-- set State Machine to first state
	SM = DETECTENGINES;
	return true;	-- false=setup failed, true=setup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
	-- this function is called each time a tag is detected
	-- Line below shows a different way to quote a string using brackets
	if (Debug() > 0) then Beep();	end;	-- sound a tone
	if (EngineNo > 0) then	-- its an engine
		print(string.format([[tag(%.2f) : Detector %d Reader %d EngineNo %d TagLocation %d Engine %s %s #%s]],
			RunTime(), Detector, Reader, EngineNo, TagLocation, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber));
		else	-- it's a car
		print(string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		end

	-- The complete packet received is in TagPacket 
	-- (some of the most used fields have already been extracted into the first parameters to tag()
	--   First 8 digits are the 4 byte Tag UID in hexidecimal
	--   Next 32 digits are the 16 bytes of Block 4 Information read from the Tag in hexidecimal
	--   Final 4 digits are the checksum and End of Packet character
	if (Debug() > 3) then print(string.format("Packet  %s", string.sub(TagPacket, 1, 40))); end
	-- index the table Tags[] by tag ID, value is the tag packet itself
	Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
	TagCount = TagCount + 1;	-- count the number of tags
	
--[[---------------------------------------------------------------------------------------]]
-- STATE MACHINE
--[[---------------------------------------------------------------------------------------]]	

--[[---------------------------------------------------------------------------------------]]	
	if (SM == IDLE) then	-- do nothing 
		return true;
		end
--[[---------------------------------------------------------------------------------------]]		
	if (SM == DETECTENGINES) then
		-- As engines run around the layout, go until all of them are detected by Detector 1 reader 0 (Aliquippa Station)
		-- Save the tag in the tag list
		--
		-- TBD - add test to be sure that we detected tags from all tag readers
		--
		-- single toot on engine rear truck detection from all detectors and readers
		if (isEngine(EngineNo) and isTagRear(TagLocation)) then
			PlaySound(41, 0, EngineNo, MyTIUNo); -- Short Toot
			end
			
		-- Schedule a random noise on engine rear truck detection on all detectors and all readers
		if (isEngine(EngineNo) and isTagRear(TagLocation)) then
			RandomAction(EngineNo, MyTIUNo);
			end
		-- Aliquippa Station	
		if (isEngine(EngineNo) and isTagFront(TagLocation) and ((isDetector(Detector, 1) and isReader(Reader, 0)))) then
			for iDex = 1, iEngineCount do
				-- EnginesUsedXXX[] is 1 based
				if (EnginesUsedNum[iDex] == EngineNo) then 	-- is the engine in the Engine List?
					if (EnginesUsedFlag[iDex] == DETECTED) then		-- has it already been detected (then all engines looped)
						-- already detected so we have detected them all
						print ("All Engines Detected");
						-- Require three engines to run this script
						if (iEngineCount == 3) then 
							SM = READYGO_3; 
							print("Three Engines Detected");
							else 
							SM = IDLE;
							print("Three Engines Not Detected, setting state machine to IDLE");
							end						
						-- speed up the engines to FIRSTSPEED  Smph for the first loop
						-- make the speed up from STARTSPEED to FIRSTSPEED  more gradual
						for jDex = 1, iEngineDetectionCount do
							SetSpeed(STARTSPEED + 3, 3, EngineDetectionOrder[jDex], MyTIUNo);
							SetSpeed(STARTSPEED + 5, 6, EngineDetectionOrder[jDex], MyTIUNo);
							SetSpeed(FIRSTSPEED , 9, EngineDetectionOrder[jDex], MyTIUNo);
							end
						-- at this point, EngineDetectionOrder[1] is just beyond Detector 1 Reader 0 (Aliquippa Station)
						Got = {false, false, false};	-- set all Got[] to false

						PFADelay = 0;
						Detector1Passenger = false;
						-- All engines have been detected and we know their order on the layout
						return true;
						end
					-- At this point, we have detected a new engine
					EnginesUsedFlag[iDex] = DETECTED;	-- flag as detected in motion - EnginesUsed[] is 0 based
					-- EngineDetectionOrder[] is 1 based
					iEngineDetectionCount = iEngineDetectionCount + 1;	-- Count number of detected engines
					EngineDetectionOrder[iEngineDetectionCount] = EngineNo;	-- remember the detection order
					print(string.format("Engine # %d detected (Index : %d) at Aliquippa", EngineNo, iEngineDetectionCount));
					BumpCounter(COUNTER01);	-- bump the Engines Detected Counter on the Program Control window
					return true;
					end
				end
			end
		-- continue in this state
		return true;
		end
--[[---------------------------------------------------------------------------------------]]
	if (SM == READYGO_3) then
		-- All engines are detected, their order is known - stop them over a reader
		-- This should pick off the engines starting at the last one. This would make
		-- sure that if two engines are between detectors, we pick them off from the last one
		--
		-- The state machine will now loop over three states, READYGO_3, READYGO_3a, and READYGO_3b.
		-- Each state will move the engine up to the next detector and then stop it there.
		--
		-- Schedule a random noise on engine rear truck detection on all detectors and all readers
		if (isEngine(EngineNo) and isTagRear(TagLocation)) then
			RandomAction(EngineNo, MyTIUNo);
			end
			
		-- run first [1] engine to Detector 3 Reader 0 (Struthers Station)
		-- Detect both front and rear tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 3) and isReader(Reader, 0)))) then
			if (EngineNo == EngineDetectionOrder[1]) then
				print(string.format("Engine # %d detected (Index : %d) at Struthers", EngineNo));
				if (Got[1]) then Throttle(0, STOPDELAY, EngineDetectionOrder[1], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[1]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[1], MyTIUNo); end
				Got[1] = true;
				end
			end
			
		-- run second [2] engine to Detector 2 Reader 0 or Reader 1 (College Station)
		-- Detect both front and rear tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 2) and isReader(Reader, 0)) or (isDetector(Detector, 2) and isReader(Reader, 1)))) then
			if (EngineNo == EngineDetectionOrder[2]) then
				print(string.format("Engine # %d detected (Index : %d) at College", EngineNo));
				if (Got[2]) then Throttle(0, STOPDELAY, EngineDetectionOrder[2], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[2]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[2], MyTIUNo); end
				Got[2] = true;
				end
			end
			
		-- run third [3] engine to Detector 1 Reader 0 (Aliquippa Station)
		-- Detect both front and read tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 1) and isReader(Reader, 0)))) then
			if (EngineNo == EngineDetectionOrder[3]) then
				print(string.format("Engine # %d detected (Index : %d) at Aliquippa", EngineNo));
				if (Got[3]) then Throttle(0, STOPDELAY, EngineDetectionOrder[3], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[3]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[3], MyTIUNo); end
				Got[3] = true;
				end
			if (isCarModel(CarModel, GP7) and isTagFront(TagLocation)) then	-- Special sounds for GP7 #1500 & #1501
				local iDly = GP7PFA(0, EngineNo, MyTIUNo);
				if (iDly > PFADelay) then	
					PFADelay = iDly;
					end
				end				
			-- set flag if a passenger train being pulled by a GP7 was detected
			if (isCarModel(CarModel, GP7)) then
				print("GP7 detected at Aliquippa");
				Detector1Passenger = true;
				else
				Detector1Passenger = false;
				end
			end

		if (Got[1] and Got[2] and Got[3]) then 	-- set speed for all engines and go to next state
--			if (isCarModel(CarModel, SW1200) and (math.random(5) == 1)) then		-- Special sounds for SW1200  #1208  Engine #4
--				local iDly = SW1200PFA(0, 4, MyTIUNo);
--				if (iDly > PFADelay) then
--					PFADelay = iDly;
--					end
--				end
			-- Counter to keep track of States
			BumpCounter(COUNTER02);		-- bump the Counter02 on the Program Control window

			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[1], MyTIUNo);	-- @Detector 3 Reader 0
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[2], MyTIUNo);	-- @Detector 2 Reader 0
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[3], MyTIUNo);	-- @Detector 1 Reader 0
			--
			-- if a passenger train is leaving Detector 1 Reader 0, set switches to diverge
			-- otherwise, set the switches to straight
			-- Both after the appropriate delay to let the trains get underway
			if (Detector1Passenger) then
				-- set the switches to diverge, Engine Number is not used but must be specified
				print("College East & West set to THROWN/DIVERGE in " .. PFADelay + 8 + 35 + 20 .. " Seconds");
				Switch(REVERSE, 1, 1, MyTIUNo, PFADelay + 8 + 35 + 20, EngineNo);  -- College East
				Switch(REVERSE, 2, 1, MyTIUNo, PFADelay + 8 + 35 + 23, EngineNo);  -- College West
				Detector1Passenger = false;
				else
				-- set the switches to straight, Engine Number is not used but must be specified
				print("College East & West set to CLOSED/STRAIGHT in " .. PFADelay + 8 + 35 + 20 .. " Seconds");
				Switch(NORMAL, 1, 1, MyTIUNo, PFADelay + 8 + 35 + 20, EngineNo);  -- College East
				Switch(NORMAL, 2, 1, MyTIUNo, PFADelay + 8 + 35 + 23, EngineNo);  -- College West
				end
			Got = {false, false, false};	-- set all Got[] to false
			SM = READYGO_3a;	-- go to next state
			PFADelay = 0;
			end
		-- continue in this state
		return true;	
		end		
--[[---------------------------------------------------------------------------------------]]		
	if (SM == READYGO_3a) then

		-- All engines are detected, their order is known - stop them over a reader
		-- Schedule a random noise on engine rear truck detection on all detectors and all readers
		if (isEngine(EngineNo) and isTagRear(TagLocation)) then
			RandomAction(EngineNo, MyTIUNo);
			end

		-- run first [1] engine to Detector 1 Reader 0 (Aliquippa Station)
		-- Detect both front and rear tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 1) and isReader(Reader, 0)))) then
			if (EngineNo == EngineDetectionOrder[1]) then
				print(string.format("Engine # %d detected (Index : %d) at Aliquippa", EngineNo));
				if (Got[1]) then Throttle(0, STOPDELAY, EngineDetectionOrder[1], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[1]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[1], MyTIUNo); end
				Got[1] = true;
				end
			if (isCarModel(CarModel, GP7) and isTagFront(TagLocation)) then	-- Special sounds for GP7 #1500 & #1501
				local iDly = GP7PFA(0, EngineNo, MyTIUNo);
				if (iDly > PFADelay) then
					PFADelay = iDly;
					end
				end
			-- set flag if a passenger train being pulled by a GP7 was detected
			if (isCarModel(CarModel, GP7)) then
				print("GP7 detected at Aliquippa");
				Detector1Passenger = true;
				else
				Detector1Passenger = false;
				end
			end

		-- run second [2] engine to Detector 3 Reader 0 (Struthers Station)
		-- Detect both front and read tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 3) and isReader(Reader, 0)))) then
			if (EngineNo == EngineDetectionOrder[2]) then
				print(string.format("Engine # %d detected (Index : %d) at Struthers", EngineNo));
				if (Got[2]) then Throttle(0, STOPDELAY, EngineDetectionOrder[2], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[2]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[2], MyTIUNo); end
				Got[2] = true;
				end
			end
		-- run third [3] engine to Detector 2 Reader 0 or Reader 1 (College Station)
		-- Detect both front and read tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 2) and isReader(Reader, 0)) or (isDetector(Detector, 2) and isReader(Reader, 1)))) then
			if (EngineNo == EngineDetectionOrder[3]) then
				print(string.format("Engine # %d detected (Index : %d) at College", EngineNo));
				if (Got[3]) then Throttle(0, STOPDELAY, EngineDetectionOrder[3], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[3]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[3], MyTIUNo); end
				Got[3] = true;
				end
			end

		if (Got[1] and Got[2] and Got[3]) then 	-- set speed for all engines and go to next state
			-- Counter to keep track of States
			BumpCounter(COUNTER02);		-- bump the Counter02 on the Program Control window
			
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[1], MyTIUNo);	-- @Detector 1 Reader 0
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[2], MyTIUNo);	-- @Detector 3 Reader 0
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[3], MyTIUNo);	-- @Detector 2 Reader 0
			--
			-- if a passenger train is leaving Detector 1 Reader 0, set switches to diverge
			-- otherwise, set the switches to straight
			-- Both after the appropriate delay to let the trains get underway
			if (Detector1Passenger) then
				-- set the switches to diverge, Engine Number is not used but must be specified
				print("College East & West set to THROWN/DIVERGE in " .. PFADelay + 8 + 35 + 20 .. " Seconds");
				Switch(DIVERGE, 1, 1, MyTIUNo, PFADelay + 8 + 35 + 20, EngineNo);  -- College East
				Switch(DIVERGE, 2, 1, MyTIUNo, PFADelay + 8 + 35 + 23, EngineNo);  -- College West
				Detector1Passenger = false;
				else
				-- set the switches to straight, Engine Number is not used but must be specified
				print("College East & West set to CLOSED/STRAIGHT in " .. PFADelay + 8 + 35 + 20 .. " Seconds");
				Switch(STRAIGHT, 1, 1, MyTIUNo, PFADelay + 8 + 35 + 20, EngineNo);  -- College East
				Switch(STRAIGHT, 2, 1, MyTIUNo, PFADelay + 8 + 35 + 23, EngineNo);  -- College West
				end
			Got = {false, false, false};	-- set all Got[] to false
			SM = READYGO_3b;	-- go to next state
			PFADelay = 0;
			end

		-- continue in this state
		return true;	
		end	
--[[---------------------------------------------------------------------------------------]]		
	if (SM == READYGO_3b) then
		-- All engines are detected, their order is known - stop them over a reader
		-- Schedule a random noise on engine rear truck detection on all detectors and all readers
		if (isEngine(EngineNo) and isTagRear(TagLocation)) then
			RandomAction(EngineNo, MyTIUNo);
			end

		-- run first [1] engine to Detector 2 Reader 0 or Reader 1 (College Station)
		-- Detect both front and rear tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 2) and isReader(Reader, 0)) or (isDetector(Detector, 2) and isReader(Reader, 1)))) then
			if (EngineNo == EngineDetectionOrder[1]) then
				print(string.format("Engine # %d detected (Index : %d) at College", EngineNo));			
				if (Got[1]) then Throttle(0, STOPDELAY, EngineDetectionOrder[1], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[1]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[1], MyTIUNo); end
				Got[1] = true;
				end
			end
			
		-- run second [2] engine to Detector 1 Reader 0 (Aliquippa Station)
		-- Detect both front and read tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 1) and isReader(Reader, 0)))) then
			if (EngineNo == EngineDetectionOrder[2]) then
				print(string.format("Engine # %d detected (Index : %d) at Aliquippa", EngineNo));
				if (Got[2]) then Throttle(0, STOPDELAY, EngineDetectionOrder[2], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[2]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[2], MyTIUNo); end
				Got[2] = true;
				end
			if (isCarModel(CarModel, GP7) and isTagFront(TagLocation)) then	-- Special sounds for GP7 #1500 & #1501
				local iDly = GP7PFA(0, EngineNo, MyTIUNo);
				if (iDly > PFADelay) then
					PFADelay = iDly;
					end				
				end				
			-- set flag if a passenger train being pulled by a GP7 was detected
			if (isCarModel(CarModel, GP7)) then
				print("GP7 detected at Aliquippa");
				Detector1Passenger = true;
				else
				Detector1Passenger = false;
				end
			end
			
		-- run third [3] engine to Detector 3 Reader 0 (Struthers Station)
		-- Detect both front and read tags as a double check to make sure we detected an engine
		if (isEngine(EngineNo) and ((isDetector(Detector, 3) and isReader(Reader, 0)))) then
			if (EngineNo == EngineDetectionOrder[3]) then
				print(string.format("Engine # %d detected (Index : %d) at Struthers", EngineNo));
				if (Got[3]) then Throttle(0, STOPDELAY, EngineDetectionOrder[3], MyTIUNo); end			-- already detected set Throttle to 0
				if (not Got[3]) then SetSpeed(0, STOPDELAY, EngineDetectionOrder[3], MyTIUNo); end
				Got[3] = true;
				end
			end

		if (Got[1] and Got[2] and Got[3]) then 	-- set speed for all engines and go to next state
			-- Counter to keep track of States
			BumpCounter(COUNTER02);		-- bump the Counter02 on the Program Control window
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[1], MyTIUNo);	-- @Detector 2 Reader 0
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[2], MyTIUNo);	-- @Detector 1 Reader 0
			SetSpeed(BASESPEED + math.random(5), PFADelay + 8 + math.random(20), EngineDetectionOrder[3], MyTIUNo);	-- @Detector 3 Reader 0
			--
			-- if a passenger train is leaving Detector 1 Reader 0, set switches to diverge
			-- otherwise, set the switches to straight
			-- Both after the appropriate delay to let the trains get underway
			if (Detector1Passenger) then
				-- set the switches to diverge, Engine Number is not used but must be specified
				print("College East & West set to THROWN/DIVERGE in " .. PFADelay + 8 + 35 + 20 .. " Seconds");
				Switch(DIVERGE, 1, 1, MyTIUNo, PFADelay + 8 + 35 + 20, EngineNo);  -- College East
				Switch(DIVERGE, 2, 1, MyTIUNo, PFADelay + 8 + 35 + 23, EngineNo);  -- College West
				Detector1Passenger = false;
				else
				print("College East & West set to CLOSED/STRAIGHT in " .. PFADelay + 8 + 35 + 20 .. " Seconds");
				-- set the switches to straight, Engine Number is not used but must be specified
				Switch(STRAIGHT, 1, 1, MyTIUNo, PFADelay + 8 + 35 + 20, EngineNo);  -- College East
				Switch(STRAIGHT, 2, 1, MyTIUNo, PFADelay + 8 + 35 + 23, EngineNo);  -- College West
				end
			Got = {false, false, false};	-- set all Got[] to false
			SM = READYGO_3;	-- go to next state
			PFADelay = 0;
			end
		-- continue in this state
		return true;
		end	
--[[---------------------------------------------------------------------------------------]]
	return true;	-- true=continue to process tags, false=stop processing tags
	end
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
	-- this function is called once when the user presses the [STOP] button
	print("cleanup()");
	for idx,Engine in ipairs(EnginesUsedNum) do		
		print(idx .. " Shutdown Engine " .. Engine);
		EngineShutdown(Engine, MyTIUNo);
		end	
	ShowTags(Engine, TIU);		-- dump out the Tags table
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]

function GP7PFA(when, EngineNo, TIUNo)
--
-- Run the GP7 PFA sequence
--
-- returns additional delay for PFA on success
--
local PFADelay = when + 38;
SetVolume(ENGINE_VOLUME, 25, when + 10, EngineNo, TIUNo);	-- reduce engine volume for the PFA sequence
PlaySound(110, when + 15, EngineNo, TIUNo); 	-- Now Boarding
PlaySound(111, when + 18, EngineNo, TIUNo); 	-- Ticketed Passengers Only
PlaySound(112, when + 21, EngineNo, TIUNo); 	-- Let me help you
PlaySound(113, when + 24, EngineNo, TIUNo); 	-- Let me take your baggage
PlaySound(114, when + 28, EngineNo, TIUNo); 	-- Thanks
PlaySound(115, when + 34, EngineNo, TIUNo); 	-- All Aboard
SetVolume(ENGINE_VOLUME, 100, when + 40, EngineNo, TIUNo);
print("GP7 PFA in " .. PFADelay .. " seconds");
return PFADelay;
end
--[[---------------------------------------------------------------------------------------]]

function SW1200PFA(when, EngineNo, TIUNo)
--
-- Run the SW1200 PFA sequence
--
-- returns additional delay for PFA on success
--
local PFADelay = when + 6 + 20 + 73;	-- its a long sequence
Startup(when + 6, EngineNo, MyTIUNo); 		-- Startup takes 20 seconds
-- when + 6 + 20
SetVolume(ENGINE_VOLUME, 25, when + 6 + 10, EngineNo, TIUNo);	-- reduce engine volume for the PFA sequence
PlaySound(80, when + 6 + 20, EngineNo, TIUNo);
PlaySound(81, when + 6 + 20 + 5, EngineNo, TIUNo);
PlaySound(82, when + 6 + 20 + 5 + 10, EngineNo, TIUNo);
PlaySound(83, when + 6 + 20 + 5 + 10 + 3, EngineNo, TIUNo);
PlaySound(84, when + 6 + 20 + 5 + 10 + 3 + 4, EngineNo, TIUNo);
PlaySound(85, when + 6 + 20 + 5 + 10 + 3 + 4 + 4, EngineNo, TIUNo);
PlaySound(86, when + 6 + 20 + 5 + 10 + 3 + 4 + 4 + 5, EngineNo, TIUNo);
PlaySound(87, when + 6 + 20 + 5 + 10 + 3 + 4 + 4 + 5 + 3, EngineNo, TIUNo);
PlaySound(88, when + 6 + 20 + 5 + 10 + 3 + 4 + 4 + 5 + 3 + 5, EngineNo, TIUNo);
PlaySound(89, when + 6 + 20 + 5 + 10 + 3 + 4 + 4 + 5 + 3 + 5 + 5, EngineNo, TIUNo);
PlaySound(90, when + 6 + 20 + 5 + 10 + 3 + 4 + 4 + 5 + 3 + 5 + 5 + 4, EngineNo, TIUNo);
Shutdown	 (when + 6 + 20 + 5 + 10 + 3 + 4 + 4 + 5 + 3 + 5 + 5 + 4 + 5, EngineNo, TIUNo);
SetVolume(ENGINE_VOLUME, 100, when + 75, EngineNo, TIUNo);
print("A&S SW1200 #1208 PFA in " .. PFADelay .. " seconds");
return PFADelay;
end
--[[---------------------------------------------------------------------------------------]]
-----------------------------------------------------------------------------
-- Schedule a Random Noise
--
-- Params:
--		EngineNo
--		TIUNo
--
-- returns true on success
--
-----------------------------------------------------------------------------
function RandomAction(iEngineNo, iTIUNo)
-- Schedule a random noise
local retv = true;

-- pick sound/effect based on a random number
-- can be adjusted if there is too much or too little background noise
--
-- choose from:
-- 1. Turn the bell on for a random number of seconds
-- 2. Blow the horn/whistle for a random number of seconds
-- 3. Play the crossing sound n42
-- 4. Turn the smoke on for a random number of seconds
-- 5. Turn on Cab Chatter for a random number of seconds
-- 6. No sounds
--
local Starttime = 0;
local Endtime = 0;
local switchval = math.random(6);	-- returns 1 through 6
if (switchval == 1) then
		Starttime = 25 + math.random(10);
		Endtime = Starttime + 5 + math.random(5);
		Bell(ON, Starttime, iEngineNo, iTIUNo); 		-- send bell on
		Bell(OFF, Endtime, iEngineNo, iTIUNo);			-- send bell off
		print("Random Action: Bell : Engine " .. iEngineNo .. " in " .. Starttime .. " seconds");
elseif (switchval == 2) then
		Starttime = 20 + math.random(10);
		Endtime = Starttime + 3 + math.random(3);
		Whistle(ON, Starttime, iEngineNo, iTIUNo); 		-- send Whistle/Horn on
		Whistle(OFF, Endtime, iEngineNo, iTIUNo);		-- send Whistle/Horn off
		print("Random Action: Horn/Whistle : Engine " .. iEngineNo .. " in " .. Starttime .. " seconds");
elseif (switchval == 3) then
		Starttime = 20 + math.random(11);
		PlaySound(42, Starttime, iEngineNo, iTIUNo); 	-- SXS Toot
		print("Random Action: SXS : Engine " .. iEngineNo .. " in " .. Starttime .. " seconds");
elseif (switchval == 4) then
		Starttime = 20 + math.random(10);
		Endtime = Starttime + 46;
		Smoke(ON, Starttime, iEngineNo, iTIUNo);		-- send Smoke on
		Smoke(OFF, Endtime, iEngineNo, iTIUNo);			-- send Smoke off
		print("Random Action: Smoke : Engine " .. iEngineNo .. " in " .. Starttime .. " seconds");
elseif (switchval == 5) then
		Starttime = math.random(10);
		if (math.random(2) == 1) then	-- math.random returns 1 or 2
			CabChat(ON, Starttime, iEngineNo, iTIUNo);		-- set Cab Chatter on
			print("Random Action: Cab Chatter On : Engine " .. iEngineNo .. " in " .. Starttime .. " seconds");
			else
			CabChat(OFF, Starttime, iEngineNo, iTIUNo);		-- set Cab Chatter off
			print("Random Action: Cab Chatter Off : Engine " .. iEngineNo .. " in " .. Starttime .. " seconds");
			end
elseif (switchval == 6) then
		-- No sounds some of the time
		print("Random Action: Quiet : Engine " .. iEngineNo);
		end
return retv;
end
--[[---------------------------------------------------------------------------------------]]

function SW1200Button(Eng, TIU)
	print("SW1200 Button()");
	SW1200PFA(0, 4, TIU);		-- only works with Engine 4, the SW1200
	return true;
end
Function01Name, Function01Label = SW1200Button, "SW 1200 PFA";
--[[---------------------------------------------------------------------------------------]]
function GP7Button(Eng, TIU)
	print("GP7 Button()");
	GP7PFA(0, 7, TIU);		-- only works with Engines 7 and 13, the GP7's
	return true;
end
Function02Name, Function02Label = GP7Button, "GP7 PFA";
--[[---------------------------------------------------------------------------------------]]
function PauseButton(Eng, TIU)
	print("Pause Button()");
	for idx,Engine in ipairs(EnginesUsedNum) do
		S1 = GetSetSpeed(Engine, MyTIUNo);	-- get Requested Speed
		print(idx .. " Pause Engine " .. Engine .. " Speed " .. S1);
		EngineSpeedSave[idx] = S1;
		SetSpeed(0, 0, Engine, MyTIUNo);
		end
	return true;
end
Function06Name, Function06Label = PauseButton, "Pause";
--[[---------------------------------------------------------------------------------------]]
function ResumeButton(Eng, TIU)
	print("Resume Button()");
	for idx,Engine in ipairs(EnginesUsedNum) do
		S1 = EngineSpeedSave[idx];		
		print(idx .. " Resume Engine " .. Engine .. " Speed " .. S1);
		SetSpeed(S1, 0, Engine, MyTIUNo);
		end
	return true;
end
Function07Name, Function07Label = ResumeButton, "Resume";
--[[---------------------------------------------------------------------------------------]]
function ShowTags(Eng, TIU)
	print("Tags detected:");
	for k,v in  pairs(Tags) do		-- dump out the Tags table
		print(k .. "  " .. v);
		end
	return true;
end
Function10Name, Function10Label = ShowTags, "Show Tags";
--[[---------------------------------------------------------------------------------------]]
