--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2022 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]

-- Semicolons are not required in Lua code

function isEngine(EngNo)
	if (EngNo > 0) then return true; end
	return false;
	end
	
function notEngine(EngNo)
	if (EngNo == 0) then return true; end
	return false;
	end	
	
function isTagRear(TagLocation)
	if (TagLocation == TAG_ON_REAR_TRUCK) then return true; end
	return false;
	end

function isTagFront(TagLocation)
	if (TagLocation == TAG_ON_FRONT_TRUCK) then return true; end
	return false;
	end
	
function isDetector(Detector, x)
	if (Detector == x) then return true; end
	return false;
	end	
	
function isReader(Reader, x)
	if (Reader == x) then return true; end
	return false;
	end	
	
function isCarModel(CarModel, x)
	if (CarModel == x) then return true; end
	return false;
	end	

--[[---------------------------------------------------------------------------------------
--
Occupancy Flags are supported as part of the Thinking Engines project.
Up to 100 flags are available to indicate block occupancy. A Zero value (UNOCCUPIED)
means the block is unoccupied. A non zero value means the block is occupied. This
is usually the Engine Number. A flag == -1 means the block was manually set occupied.

The Layout window must be visible to use these functions and the flags themselves must
be defined on that Layout window.
--
]]
--[[---------------------------------------------------------------------------------------]]
--ClearAllOccupancy
-- Clear all occupancy flags to zero

function ClearAllOccupancy()
local block;
for block = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(block, UNOCCUPIED);
	end;
return;
end;

--[[---------------------------------------------------------------------------------------]]
--SetOccupancy
-- Set a value into the block occupancy flag
-- Waits 'forever' for the block to show as unoccupied before it sets occupancy
-- returns two values
-- 1: true on success, false on failure
-- 2: on failure, the block numbers that would not lock, otherwise nil
-- Wait is the number of seconds to wait, default to 1800 seconds (30 minutes)
-- When occupied, the Flag should be set to the engine number of the engine in the block.

function SetOccupancy(block, value, Wait)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in SetOccupancy()");
	end;
if (value == 0) then
	return EmergencyStop("Value in SetOccupancy() is zero, use ClearOccupancy() instead");
	end;
local WaitTime = Wait or 1800;	-- let's say that 30 minutes is 'forever'
-- set the occupancy flag if it is zero or we already have ownership - use lock
while (not TrySetFlag(block, value)) do
	-- wait 'forever'
	if (((WaitTime >> 1) % 10) == 0) then
		print("Waiting for " .. BlockName[block] .. " to clear");
		end;
	Sleep(2);
	WaitTime = WaitTime - 2;
	if (WaitTime <= 0) then
		return EmergencyStop("SetOccupancy() Timed out waiting for " .. BlockName[block] .. " to clear"); 
		end	
	end;
print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- TrySetOccupancy
-- Tries to set a value into the block occupancy flag
-- Only tries once. Returns true on success, false if the block is occupied.

function TrySetOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in TrySetOccupancy()");
	end;
if (value == 0) then
	return EmergencyStop("Value in TrySetOccupancy() is zero, use ClearOccupancy() instead");
	end;
-- set the occupancy flag if it is zero or we already have ownership - use lock
if (not TrySetFlag(block, value)) then
	-- did not get a lock
	print("TrySetOccupancy() flag on " .. BlockName[block] .. " - could not get lock");
	return false;
	end;
print("TrySetOccupancy() flag on " .. BlockName[block] .. " to " .. value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- ClearOccupancy
-- Clear a block occupancy flag to zero if we have ownership, that is,
-- we are occupying the block.

function ClearOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in ClearOccupancy()");
	end;
if (value ~= nil and GetFlag(block) == value) then
	print("ClearOccupancy() flag on " .. BlockName[block]);
	SetFlag(block, 0);
	end;
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- OverrideOccupancy
-- set a block occupancy flag without waiting for lock or checking ownership
-- Can be very dangerous if your script makes a mistake.

function OverrideOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in OverrideOccupancy()");
	end;
if (value == nil) then
	value = 0;
	end;
print("OverrideOccupancy() flag on " .. BlockName[block] .. " to " .. value);
SetFlag(block, value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- GetOccupancy
-- fetch a block occupany flag

function GetOccupancy(block)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return EmergencyStop("Invalid Block Number in GetOccupancy()");
	end;
return GetFlag(block);
end;
--[[---------------------------------------------------------------------------------------]]
-- MultiSetOccupancy
-- Set a value into several block occupancy flags. All blocks must be unoccupied.
-- Waits 1800 seconds or the given 'Wait' time in seconds for the blocks to show as unoccupied before it sets occupancy
-- Returns two values
-- 1: true on success, false on failure
-- 2: on failure, one of the block numbers that would not lock, otherwise 0
-- Wait is the number of seconds to wait, default to 1800 (30 minutes)
-- blocks is a table of the form {2, 44, 21}

function MultiSetOccupancy(blocks, value, Wait)
if (type(blocks) ~= "table") then return EmergencyStop("MultiSetOccupancy() block not a table"), 0; end
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	if (v < FIRST_BLOCK or v > LAST_BLOCK) then
		return EmergencyStop("Invalid Block Number in MultiSetOccupancy()"), 0;
		end;
	end;
if (value == 0) then
	return EmergencyStop("Value in MultiSetOccupancy() is zero, use ClearOccupancy() instead"), 0;
	end;
local WaitTime = Wait or 1800;	-- let's say 30 minutes is 'forever'
-- set the occupancy flag if it is zero or we already have ownership - use lock
local occupiedblock;
while (true) do
	local result;
	result, occupiedblock = TryMultiSetFlag(blocks, value)
	if (result) then
		break;
		end;
	-- wait almost forever
	if (((WaitTime >> 1) % 10) == 0) then
		print("MultiSetOccupancy() Waiting for " .. BlockName[occupiedblock] .. " to clear");
		end;
	Sleep(2);
	WaitTime = WaitTime - 2;
	if (WaitTime <= 0) then
		return EmergencyStop("MultiSetOccupancy() Timed out waiting for " .. BlockName[occupiedblock] .. " to clear"); 
		end
	end;
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	print("MultiSetOccupancy() flag on " .. BlockName[v] .. " to " .. value);
	end;
return true, 0;
end;
--[[---------------------------------------------------------------------------------------]]
-- TryMultiSetOccupancy
-- Set a value into several block occupancy flags. All blocks must be unoccupied.
-- Only tries once
-- returns two values
-- 1: true on success, false on failure
-- 2: on failure, the block number that would not lock, otherwise nil

function TryMultiSetOccupancy(blocks, value)
-- blocks is a table of the form {2, 44, 21}
if (type(blocks) ~= "table") then return EmergencyStop("TryMultiSetOccupancy() block not a table"); end
for k, v in ipairs(blocks) do
	if (v < FIRST_BLOCK or v > LAST_BLOCK) then
		return EmergencyStop("Invalid Block Number in TryMultiSetOccupancy()"), nil;
		end;
	end;
if (value == 0) then
	return EmergencyStop("Value in TryMultiSetOccupancy() is zero, use ClearOccupancy() instead"), nil;
	end;
-- set the occupancy flag if it is zero or we already have ownership - use lock
local success, occupiedblock = TryMultiSetFlag(blocks, value);
if (not success) then
	-- did not get a lock
	print("TryMultiSetOccupancy() flag on " .. BlockName[occupiedblock] .. " - could not get lock");
	return false, occupiedblock;
	end;
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	print("TryMultiSetOccupancy() flag on " .. BlockName[v] .. " to " .. value);
	end;
return true, nil;
end;
--[[---------------------------------------------------------------------------------------]]
-- SingleSetOccupancy
-- Set a value into one of several block occupancy flags
-- Waits 1800 seconds or the given 'Wait' time in seconds for one of the blocks to show as unoccupied before it sets occupancy
-- returns two values
-- 1: true on success, false on failure
-- 2: on success, the block number that locked, otherwise nil
-- Wait is the number of seconds to wait, default to 1800 (30 minutes)
-- blocks is a table of the form {2, 44, 21} or {2}

function SingleSetOccupancy(blocks, value, Wait)
if (type(blocks) ~= "table") then return EmergencyStop("SingleSetOccupancy() block not a table"), 0; end
-- loop through all blocks in table
for k, v in ipairs(blocks) do
	if (v < FIRST_BLOCK or v > LAST_BLOCK) then
		return EmergencyStop("Invalid Block Number in SingleSetOccupancy()"), nil;
		end;
	end;
if (value == 0) then
	return EmergencyStop("Value in SingleSetOccupancy() is zero, use ClearOccupancy() instead"), nil;
	end;
local WaitTime = Wait or 1800;	-- let's say that 30 minutes is 'forever'
-- set the occupancy flag if it is zero or we already have ownership - use lock
local occupiedblock;
while (true) do
	for k,v in ipairs(blocks) do
		-- lock the first block that becomes available
		local result = TrySetFlag(v, value)
		if (result) then
			occupiedblock = v;
			print("SingleSetOccupancy() flag on " .. BlockName[occupiedblock] .. " to " .. value);
			return true, occupiedblock;
			end;
		if (((WaitTime >> 1) % 10) == 0) then
			print("SingleSetOccupancy() Waiting for " .. BlockName[v] .. " to clear");
			end;
		end;
	-- wait almost forever
	Sleep(2);
	WaitTime = WaitTime - 2;
	if (WaitTime <= 0) then
		break;
		end		
	end;

	for k,v in ipairs(blocks) do
		return EmergencyStop("SingleSetOccupancy() Timed out waiting for " .. BlockName[v] .. " to clear"); 
		end;

return false, nil;
end;
--[[---------------------------------------------------------------------------------------]]

-- from   https://q-syshelp.qsc.com/DeveloperHelp/Content/Code_Examples/Hex_Data_Handling.htm

--Convert a string's bytes to a string of human readable hex encoded value pairs
function hex2str(str)
  return str:gsub(".",
    function(byte)
      return string.format("%02X", string.byte(byte))
    end
  )
end

--Translate a string of 2 character hex values (human readable) to a string of character encoded bytes
function str2hex(str)
  return str:gsub("..", 
    function(byte) 
      return string.char( tonumber(byte,16) ) 
    end
  )
end

--Return a numerical equivalent of the bytes within the string
--Note that a maximum of 7 bytes can be converted by this function before the number will overflow
function hex2num(str)
  local value = 0
  if str:len()>0 then
    for i=1,str:len() do
      value = str:byte(i) + (value << 8)
    end
  end
  return value
end

--Return a string of bytes equivalent to the numerical value
function num2hex(val)
  local str = ""
  while val > 0 do
    str = string.char(val % 256) .. str 
    val = val >> 8
  end
  return str
end

--[[
Handling Hex and Binary Data

Lua scripts often encounter hex or binary data coming from remote devices. Handling this 
data effectively is achieved by planning the context of how the data is formatted, which 
avoids inconsistencies and confusion.

Received data from the TCPSocket, UDPSocket, and Serial objects is formatted as ASCII characters 
and wrapped in a string. The data is stored sequentially as received and each character 
represents one byte (0-255 decimal).

A byte value encoded as a character in a string can be input as a hex-encoded escape character
, a decimal value escape character, a character, or encoded from a number using the string.char
 function:

“\x41” == “\65” == ”A” = string.char(65) == string.char(0x41)

The same byte can be encoded as a number using hex notation or decimal notation, translated 
from a character to a numerical byte:

0x41 == 65 == string.byte(“A”)

Translating this byte data into a readable format (strings or numbers) is a common task. This 
allows the non-printable characters in the string of bytes to be used more effectively. The 
following helper functions support these tasks.

--String conversion test
local data = "\x48\x65\x6C\x6C\x6F\x20\x57\x6F\x72\x6C\x64\x21"
print(data)           --> Hello World!
data = hex2str(data)
print(data)           --> 48656C6C6F20576F726C6421
print(str2hex(data))  --> Hello World!

--Number conversion test
data = "\xDE\xCA\xFB\xAD"
print(data)           --> ????   (4 unprintable characters)
print(hex2str(data))  --> DECAFBAD
data = hex2num(data)
print(data)           --> 3737844653
data = num2hex(data)
print(hex2str(data))  --> DECAFBAD
]]
--[[---------------------------------------------------------------------------------------]]
